<?php
namespace Controller;

use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\PaymentPlatform\Implementations\SoftwareExpress\PaymentImpl;
use Zeedhi\PaymentPlatform\InstantBuy\InstantBuyPlatform;
use Zeedhi\PaymentPlatform\InstantBuy\InstantBuyTransaction;
use Zeedhi\PaymentPlatform\Customer;
use Zeedhi\PaymentPlatform\CreditCard;

class AutomaticRecharge extends \Zeedhi\Framework\Controller\Simple {

    private $automaticRechargeService;
    private $messagesService;
    private $smtpService;
    private $instanceManager;
    private $entityManager;
    private $evoluiSaldoService;

    public function __construct(\Service\AutomaticRecharge $automaticRechargeService, \Service\MessagesService $messagesService, \Service\SMTP $smtpService, \Zeedhi\Framework\DependencyInjection\InstanceManager $instanceManager, \Doctrine\ORM\EntityManager $entityManager, \Service\EvoluiSaldo $evoluiSaldoService) {
        $this->automaticRechargeService = $automaticRechargeService;
        $this->messagesService = $messagesService;
        $this->smtpService = $smtpService;
        $this->instanceManager = $instanceManager;
        $this->entityManager = $entityManager;
        $this->evoluiSaldoService = $evoluiSaldoService;
    }

    public function runAutomaticRecharge() {
        try {

            $date = date("d-m-Y H:i:s");

            $this->messagesService->writeLog("\n" . $date . " - Iniciando Recargas Automaticas.\n");

            $consumers = $this->automaticRechargeService->getConsumers();
            
            $orderCustomers = [];
            foreach ($consumers as $consumer){
                $orderCustomers[$consumer["CARTCREDMD5"]][] = $consumer;
            }
            
            if(isset($orderCustomers[""])){
                foreach($orderCustomers[""] as $noCardCustomer){
                    $cpf = $this->automaticRechargeService->getConsumerCPF($noCardCustomer['CDCLIENTE'], $noCardCustomer['CDCONSUMIDOR']);
                    $cardInfo = $this->automaticRechargeService->getCardInfo($cpf['NRCPFRESPCON']);
                    $noCardCustomer['DSTOKEN'] = $cardInfo['DSTOKEN'];
                    $noCardCustomer['CARTCREDMD5'] = $cardInfo['CARTCREDMD5'];
                    $noCardCustomer['NRULTNUM'] = $cardInfo['NRULTNUM'];
                    $noCardCustomer['IDESITEF'] = $cardInfo['IDESITEF'];
                    $noCardCustomer['CDTIPORECE'] = $cardInfo['CDTIPORECE'];
                    $noCardCustomer['NMTITULAR'] = $cardInfo['NMTITULAR'];
                    $orderCustomers[$noCardCustomer["CARTCREDMD5"]][] = $noCardCustomer;
                }   
            }
            
            unset($orderCustomers[""]);
            $rechargeNumber = 0;

            foreach ($orderCustomers as $orderCustomersInformations){
                foreach ($orderCustomersInformations as $chargeFamily){
                    $rechargeNumber++;
                }
            }

            $this->messagesService->writeLog($rechargeNumber . " recarga(s) para realizar.\n");

            $processedNum = 0;
            foreach ($orderCustomers as $orderCustomersInformations){
                try {
                    $this->messagesService->writeLog("iniciando recarga para CDCLIENTE: " . $orderCustomersInformations[0]['CDCLIENTE'] . ", CDCONSUMIDOR: " . $orderCustomersInformations[0]['CDCONSUMIDOR'] . "\n");

                    $rechargeValue = 0;
                    foreach ($orderCustomersInformations as $chargeFamily){
                        $this->messagesService->writeLog("Família: " . $chargeFamily["CDFAMILISALD"] . ", Saldo atual: " . $chargeFamily["VRSALDCONFAM"] . ", Valor mínimo: " . $chargeFamily["VRMINSALDORRECAR"] . ", Valor da recarga: " . $chargeFamily["VRRECARGARAUT"] . "\n");

                        if (empty($chargeFamily['IDESITEF'])){
                            throw new \Exception("Código de bandeira não cadastrado para o cartão do cliente. (". $chargeFamily["CDCLIENTE"] ."->" . $chargeFamily["CDCONSUMIDOR"] . ")");
                        }
                        if (empty($chargeFamily['CDTIPORECE'])){
                            throw new \Exception("Bandeira não vinculada à um tipo de recebimento. (". $chargeFamily["CDCLIENTE"] ."->" . $chargeFamily["CDCONSUMIDOR"] . ")");
                        }
                        $rechargeValue += $chargeFamily['VRRECARGARAUT'];
                    }

                    $NRTRANSACAO = $this->automaticRechargeService->newCode('TRANSACAO', 12);
                    $this->automaticRechargeService->insertTransacao($NRTRANSACAO, $orderCustomersInformations[0]['CDCLIENTE'], $orderCustomersInformations[0]['CDCONSUMIDOR'], $rechargeValue, $orderCustomersInformations[0]['NRULTNUM']);
                    $this->automaticRechargeService->insertTransacaoCons($NRTRANSACAO, $orderCustomersInformations[0]['CDCLIENTE'], $orderCustomersInformations[0]['CDCONSUMIDOR']);

                    $paymentInfo = array(
                        "CDCLIENTE" => $orderCustomersInformations[0]['CDCLIENTE'],
                        "CONSUMERID" => $orderCustomersInformations[0]['CDCONSUMIDOR'],
                        "DSTOKEN" => $orderCustomersInformations[0]['DSTOKEN'],
                        "NMTITULAR" => $orderCustomersInformations[0]['NMTITULAR'],
                        "IDESITEF" => $orderCustomersInformations[0]['IDESITEF'],
                        "RECHARGEVALUE" => $rechargeValue
                    );

                    $this->automaticRechargeService->updateTransacao($NRTRANSACAO, "Processando Pagamento da RA.", null);

                    $transactionId = $this->rechargePayment($paymentInfo);

                    $this->automaticRechargeService->updateTransacao($NRTRANSACAO, "Recarga automática aprovada. Inserindo créditos.", $transactionId['nsu']);

                    $beforeProcessedNum = $processedNum;

                    try{
                        $this->entityManager->getConnection()->beginTransaction();

                        foreach($orderCustomersInformations as $orderCustomerInformations){
                            $this->recharge($orderCustomerInformations, $transactionId);
                            sleep(1);
                            $processedNum++;
                        }

                        $captureResponse = $this->capturePayment($transactionId, $paymentInfo);
                        $this->entityManager->getConnection()->commit();
                        $this->automaticRechargeService->updateTransacao($NRTRANSACAO, "Recarga automática finalizada com sucesso.", $transactionId['nsu']);
                    } catch (\Exception $e){
                        $this->entityManager->getConnection()->rollBack();

                        $processedNum = $beforeProcessedNum;

                        $message = substr($e->getMessage(),0,255);
                        $this->automaticRechargeService->updateTransacao($NRTRANSACAO, "Erro na inserção de crédito da RA. Erro: " . $message, $transactionId['nsu']);
                        throw new \Exception($message);
                    }
                } catch (\Exception $e){
                    $userDetails = "CDCLIENTE: " . $orderCustomersInformations[0]['CDCLIENTE'] . ", CDCONSUMIDOR: " . $orderCustomersInformations[0]['CDCONSUMIDOR'];
                    if (sizeof($orderCustomersInformations) > 1){
                        $userDetails .= " e " . (sizeof($orderCustomersInformations)-1) . " associado(s) - ";
                    }
                    else $userDetails .= " - ";
                    $this->messagesService->writeLog($userDetails . $e->getMessage() . "\n");
                    $this->automaticRechargeService->updateTransacao($NRTRANSACAO, "Erro no processamento do pagamento da RA. Erro: " . $e->getMessage(), null);
                }
            }
            $this->messagesService->writeLog($processedNum . " recargas automaticas processadas.\n\n\n");
            try {
                //$this->mailLog();
            } catch(\Exception $e) {
                $this->messagesService->writeLog("Não foi possível enviar o e-mail de log da RA para a administração. Erro" . $e->getMessage());
            }
        } catch(\Exception $e){
            $this->messagesService->writeLog($e->getMessage());
        }
    }

    private function rechargePayment($paymentInfo){
        $filial      = $this->automaticRechargeService->getFilial($paymentInfo["CDCLIENTE"]);
        $NRTRANSACAO = $this->automaticRechargeService->newCode('TRANSACAO', 12);
        return $this->paymentService($paymentInfo, $filial["CDLOJAESITEFFIL"], $filial["IDLOJAESITEFFIL"], $NRTRANSACAO);
    }

    private function capturePayment($transactionId, $paymentInfo){
        $filial      = $this->automaticRechargeService->getFilial($paymentInfo["CDCLIENTE"]);
        return $this->captureService($filial["CDLOJAESITEFFIL"], $filial["IDLOJAESITEFFIL"], $transactionId);
    }

    private function recharge($consumer, $transactionId){
        // this will be outside this function now
        //$NRTRANSACAO = $this->automaticRechargeService->newCode('TRANSACAO', 12);
        //$this->automaticRechargeService->insertTransacao($NRTRANSACAO, $consumer["CDCLIENTE"], $consumer["CDCONSUMIDOR"], $consumer['VRRECARGARAUT'], $consumer['NRULTNUM']);
        //$this->automaticRechargeService->insertTransacaoCons($NRTRANSACAO, $consumer["CDCLIENTE"], $consumer["CDCONSUMIDOR"]);
        $this->transactionSuccess($consumer, $transactionId);
    }

    private function paymentService($paymentInfo, $CDLOJAESITEFFIL, $IDLOJAESITEFFIL, $NRTRANSACAO){
        $paymentPlatform = PaymentImpl::factory($CDLOJAESITEFFIL, $IDLOJAESITEFFIL, PaymentImpl::PRODUCTION);

        $customer = new Customer();
        $customer->setId(substr(ltrim($paymentInfo["CONSUMERID"], '0'), '0', '20'));
        $customer->setName($paymentInfo["NMTITULAR"]);
        $transaction = new InstantBuyTransaction(ceil($paymentInfo["RECHARGEVALUE"] * 100), $NRTRANSACAO, $paymentInfo["DSTOKEN"], $customer);
        $transaction->setFlag($this->decodeFlag($paymentInfo["IDESITEF"]));

        return $paymentPlatform->instantBuyTransaction($transaction);
    }

    private function captureService($CDLOJAESITEFFIL, $IDLOJAESITEFFIL, $transactionId){
        $paymentPlatform = PaymentImpl::factory($CDLOJAESITEFFIL, $IDLOJAESITEFFIL, PaymentImpl::PRODUCTION);

        $captureResponse = $paymentPlatform->captureTransaction($transactionId['captureUrl'], $transactionId['paymentId']);

        return $captureResponse;
    }

    private function decodeFlag($flag){
        switch($flag){
            case '1':
                return CreditCard::FLAG_VISA;
            case '2':
                return CreditCard::FLAG_MASTERCARD;
            case '3':
                return CreditCard::FLAG_AMEX;
            case '5':
                return CreditCard::FLAG_HIPERCARD;
            case '6':
                return CreditCard::FLAG_AURA;
            case '33':
                return CreditCard::FLAG_DINERS;
            case '41':
                return CreditCard::FLAG_ELO;
            case '224':
                return CreditCard::FLAG_ALELO;
            case '281':
                return CreditCard::FLAG_SODEXO;
            default:
                throw new \Exception("Bandeira invalida!");
        }
    }

    private function transactionSuccess($consumer, $transactionId){
        $isFirstInsertion = $this->automaticRechargeService->isFirstInsertion($consumer['CDCLIENTE'], $consumer['CDCONSUMIDOR'], $consumer['CDFAMILISALD']);
        $currentBalance = $this->automaticRechargeService->getCurrentBalance($consumer['CDCLIENTE'], $consumer['CDCONSUMIDOR'], $consumer['CDFAMILISALD']);
        $valueToRecharge = floatval(str_replace(',', '.', $consumer['VRRECARGARAUT']));

        $recalcParam = $this->instanceManager->getParameter("recalculateMethod");

        if ($valueToRecharge > 0) {
            $balance = $this->automaticRechargeService->checkBalance($consumer['CDCLIENTE'], $consumer['CDCONSUMIDOR'], $consumer['CDFAMILISALD']);
            $VRSALDCONEXT = $valueToRecharge;
            if (!empty($isFirstInsertion)){
                if (!empty($currentBalance)){
                    $VRSALDCONEXT += floatval($currentBalance[0]['VRSALDCONEXT']);
                }
            }
            $CDFILIAL = $this->automaticRechargeService->getFilial($consumer['CDCLIENTE']);
            $CDFILIAL = $CDFILIAL['CDFILIAL'];
            $this->automaticRechargeService->insertExtratoCons($consumer['CDCLIENTE'], $consumer['CDCONSUMIDOR'], $consumer['CDFAMILISALD'], 'Crédito No Cash (RA)', $valueToRecharge, 'C', $VRSALDCONEXT, $consumer['CDTIPORECE'], $CDFILIAL);
            if (empty($balance)){
                $this->automaticRechargeService->createBalance($consumer['CDCLIENTE'], $consumer['CDCONSUMIDOR'], $consumer['CDFAMILISALD'], $valueToRecharge);
            } else {
                $this->automaticRechargeService->updateBalance($consumer['CDCLIENTE'], $consumer['CDCONSUMIDOR'], $consumer['CDFAMILISALD'], $VRSALDCONEXT);
            }

            //Racalculate balance process
            if($recalcParam === 'F'){
                $evoluiSaldoDate = $this->evoluiSaldoService->getTransactionDate($recalcParam, $consumer['CDCLIENTE'], $consumer['CDCONSUMIDOR']);
                $NRORG = $this->evoluiSaldoService->getConsumerNrorg($consumer['CDCLIENTE'], $consumer['CDCONSUMIDOR']);
                
                $conditions = array(
                    'CDCLIENTE' => $consumer['CDCLIENTE'],
                    'CDCONSUMIDOR' => $consumer['CDCONSUMIDOR'],
                    'DTMOVEXTCONS' => $evoluiSaldoDate['DTMOVEXTCONS'],
                    'NRORG' => $NRORG['NRORG']
                );

                $updateSaldo = $this->evoluiSaldoService->EvoluiSaldoFunction($conditions);
            } else if($recalcParam === 'P'){
                $evoluiSaldoDate = $this->evoluiSaldoService->getTransactionDate($recalcParam, $consumer['CDCLIENTE'], $consumer['CDCONSUMIDOR'], $consumer['CDFAMILISALD']);

                $this->evoluiSaldoService->recalculateBalance($consumer['CDCLIENTE'], $consumer['CDCONSUMIDOR'], $consumer['CDFAMILISALD'], $evoluiSaldoDate['DTMOVEXTCONS']);
            }
        }

        // this will be outside this function now
        //$this->automaticRechargeService->updateTransacao($NRTRANSACAO, "Recarga automática concluída com sucesso. Processamento concluído", $transactionId['nsu']);
        try {
            if($consumer['HABENVEMAIL'] === 'S'){
                $this->sendEmail($consumer);
            }
        } catch(\Exception $e) {
            $this->messagesService->writeLog("Não foi possível enviar o e-mail para o consumidor CDCLIENTE: " . $consumer['CDCLIENTE'] . " -- CDCONSUMIDOR: " .$consumer['CDCONSUMIDOR'] . "Erro: " . $e->getMessage() . "\n\n\n");
        }
    }

    private function sendEmail($consumer){
        $consumerDetails = $this->automaticRechargeService->getConsumerDetails($consumer['CDCLIENTE'], $consumer['CDCONSUMIDOR']);
        $codigoCartao = "XXXX XXXX XXXX ".$consumer['NRULTNUM'];

        $valorPedido = number_format($consumer["VRRECARGARAUT"], 2);
        $valorPedido = str_replace(".", ",", $valorPedido);
        $nomeDestinatario = $consumerDetails['NMRESPCONS'];
        $emailDestinatario = $consumerDetails['DSEMAILCONS'];

        $subject = $this->messagesService->getMessage('24');
        $sender = $this->messagesService->getMessage('14');
        $content =
            '<p><b>Nome do Respons&aacute;vel: </b>'.$nomeDestinatario.'<br>'.
            '<b>Nome do Aluno: </b>'.$consumerDetails['NMCONSUMIDOR'].'<br>'.
            '<b>C&oacute;digo do Cart&atilde;o: </b>'.$codigoCartao.'</p>'.
            '<b>Valor Total da Recarga:</b> R$ '.$valorPedido.'</p>'.
            '<b>Data da Transa&ccedil;&atilde;o: </b>'. date("d/m/Y H:i:s") .'</p>'
        ;

        $this->smtpService->sendEmail($nomeDestinatario, $emailDestinatario, $sender, $subject, $content);
    }

    private function mailLog(){
        $log = file_get_contents("../mailing_log.txt", FILE_TEXT);
        $log = str_replace("\n", "<br>", $log);
        $email = $this->instanceManager->getParameter("log_email");
        $this->smtpService->sendEmail("Admin No Cash", $email, "No Cash App", "Logs da Recarga Automática No Cash", $log, false);
        file_put_contents("../mailing_log.txt", "");
    }

}
