<?php
/*
	author: Luis Philipe Fidelis
	  date: March, 2016
    e-mail: luis.silva@teknisa.com
*/

namespace Odhen\API\Service;

use NFePHP\NFe\ToolsNFe;
use NFePHP\Common\Files\FilesFolders;
use Zeedhi\Framework\DependencyInjection\InstanceManager;

class NotaFiscal {

    const APROVADA = 'O';
    const ENTRADA = 'E';
    const SAIDA = 'S';
    const PRODUTO_NACIONAL = 'N';

	protected $entityManager;
	protected $util;
	protected $databaseUtil;
	protected $impressaoNFCE;

    public function __construct(\Doctrine\ORM\EntityManager $entityManager,
    							\Odhen\API\Util\Util $util,
								\Odhen\API\Util\Database $databaseUtil,
								\Odhen\API\Service\ImpressaoNFCE $impressaoNFCE){
	   	$this->entityManager 	 = $entityManager;
	   	$this->util 			 = $util;
	   	$this->databaseUtil 	 = $databaseUtil;
		$this->impressaoNFCE 	 = $impressaoNFCE;
	}

	private function validaDadosEmpresa($cdfilial, $nrorg){
	    $getEmpresaParams = array(
	        'CDFILIAL' => $cdfilial,
	        'NRORG'    => $nrorg
    	);
		$result = $this->entityManager->getConnection()->fetchAll(self::GET_EMPRESAFILIAL,$getEmpresaParams);
		$arrayResult = array();
		if(isset($result[0])){
			$arrayResult['error'] = false;
			$arrayResult['dadosEmpresa'] = $result[0];
		} else {
			$arrayResult['error'] = true;
			$arrayResult['message'] = "Não há dados para a filial informada";
		}
		return $arrayResult;
	}

	const GET_EMPRESAFILIAL = "
      SELECT F.CDINSCESTA, E.CDEMPRESA, E.CDPAIS, E.SGESTADO, E.CDMUNICIPIO, F.NRINSJURFILI, E.NMRAZSOCEMPR
        FROM FILIAL F, EMPRESA E
       WHERE F.CDFILIAL = :CDFILIAL
         AND F.NRORG    = :NRORG
         AND E.NRORG    = F.NRORG
         AND E.CDEMPRESA = F.CDEMPRESA
    ";

    private function validaEnderecoFilial($cdfilial, $nrorg){
 		$getEmpresaParams = array(
	        'CDFILIAL' => $cdfilial,
	        'NRORG'    => $nrorg
    	);
    	$result = $this->entityManager->getConnection()->fetchAll(self::GET_ENDEFILI,$getEmpresaParams);
    	$arrayResult = array();
		if(isset($result[0])){
			$arrayResult['error'] = false;
			$arrayResult['dadosEndereco'] = $result[0];
		} else {
			$arrayResult['error'] = true;
			$arrayResult['message'] = "Não há endereco para a filial logada";
		}
		return $arrayResult;
    }

    const GET_ENDEFILI = "
        SELECT SGESTADO, CDMUNICIPIO, CDPAIS
           FROM ENDEFILI
          WHERE CDFILIAL     = :CDFILIAL
            AND IDTPENDEFILI = 'P'
            AND NRORG        = :NRORG
    ";

    private function validaDadosEmitenteXML($cdfilial, $nrorg){
		$dadosEmitenteParams = array(
	        'CDFILIAL' => $cdfilial,
			'NRORG'    => $nrorg
	  	);
    	$result = $this->entityManager->getConnection()->fetchAll(self::GET_DADOSEMITENTE_IDE_XML,$dadosEmitenteParams);
    	$arrayResult = array();
		if(isset($result[0])){
			$arrayResult['error'] = false;
			$arrayResult['dadosEmitente'] = $result[0];
		} else {
			$arrayResult['error'] = true;
			$arrayResult['message'] = "Não há dados de emitente da filial.";
		}
		return $arrayResult;
    }

    const GET_DADOSEMITENTE_IDE_XML = "
        SELECT F.NRINSJURFILI, UPPER(F.NMRAZSOCFILI) NMRAZSOCFILI, UPPER(F.NMFILIAL) NMFILIAL, UPPER(EF.DSENDEFILI) DSENDEFILI,
               EF.NRENDEFILISEP,UPPER(EF.NMBAIRFILI) NMBAIRFILI,M.CDMUNICIBGE,UPPER(M.NMMUNICIPIO) NMMUNICIPIO, EF.SGESTADO,
               EF.NRCEPFILI, P.CDPAISBACEN,UPPER(P.NMPAIS) NMPAIS,F.CDINSCESTA,F.CDINSCMUNI,E.CDCNAE,PE.CDSITUCRT, ES.CDESTADOIBGE
          FROM FILIAL F, ENDEFILI EF, MUNICIPIO M, PAIS P, PARAMEMPRESA PE,
               EMPRESA E, ESTADO ES
         WHERE F.CDFILIAL      = :CDFILIAL
           AND F.NRORG         = :NRORG
           AND EF.CDFILIAL     = F.CDFILIAL
           AND EF.NRORG        = F.NRORG
           AND EF.IDTPENDEFILI = 'P'
           AND ES.SGESTADO     = EF.SGESTADO
           AND ES.CDPAIS       = EF.CDPAIS
           AND M.CDMUNICIPIO   = EF.CDMUNICIPIO
           AND M.SGESTADO      = EF.SGESTADO
           AND P.CDPAIS        = EF.CDPAIS
           AND E.CDEMPRESA     = F.CDEMPRESA
           AND E.NRORG         = F.NRORG
           AND PE.CDEMPRESA    = E.CDEMPRESA
           AND PE.NRORG        = E.NRORG
    ";

    private function validaSerieNFCE($filialInfo){

    	$serieNfceParams = array(
	        'CDFILIAL' => $filialInfo['cdfilial'],
			'CDCAIXA'  => $filialInfo['cdcaixa'],
			'NRORG'    => $filialInfo['nrorg']
		);
		$result = $this->entityManager->getConnection()->fetchAll(self::GET_SERIE_NFCE,$serieNfceParams);

		$arrayResult = array();
		if(isset($result[0])){
			$arrayResult['error'] = false;
			$arrayResult['dadosSerie'] = $result[0];
		} else {
			$arrayResult['error'] = true;
			$arrayResult['message'] = "Não há serie NFCE no caixa logado.";
		}
		return $arrayResult;
    }


     const GET_SERIE_NFCE = "
        SELECT CDSERIECX, CDCAIXA
          FROM CAIXA
         WHERE CDFILIAL = :CDFILIAL
           AND CDCAIXA  = :CDCAIXA
           AND NRORG    = :NRORG
    ";

    public function montaArrayFilial($dadosEmitente, $dadosEmpresaResult, $endeFiliResult, $nrorg, $cdcaixa, $cdfilial, $cdloja, $cdoperador){
    	return array(
    		'nmrazsocfili' => $dadosEmitente['NMRAZSOCFILI'],
		    'nmfilial' => $dadosEmitente['NMFILIAL'],
		    'nrinsjurfili' => $dadosEmitente['NRINSJURFILI'],
		    'cdestadoibge' => $dadosEmitente['CDESTADOIBGE'],
		    'cdmunicibge' => $dadosEmitente['CDMUNICIBGE'],
		    'cdinscestafili' => $dadosEmitente['CDINSCESTA'],
		    'cdinscmunifili' => $dadosEmitente['CDINSCMUNI'],
		    'dsendefili' => $dadosEmitente['DSENDEFILI'],
		    'nrendefilisep' => $dadosEmitente['NRENDEFILISEP'] ? $dadosEmitente['NRENDEFILISEP'] : '.',
		    'nmbairfili' => $dadosEmitente['NMBAIRFILI'],
		    'nmmunicipio' => $dadosEmitente['NMMUNICIPIO'],
		    'sgestadofili' => $dadosEmitente['SGESTADO'],
		    'nrcepfili' => str_replace('-','',$dadosEmitente['NRCEPFILI']),
		    'cdpaisbacen' => $dadosEmitente['CDPAISBACEN'],
		    'nmpais' => $dadosEmitente['NMPAIS'],
		    'cdcnae' => $dadosEmitente['CDCNAE'],
		    'cdsitucrt'  => $dadosEmitente['CDSITUCRT'],
		    'cdempresa' => $dadosEmpresaResult['CDEMPRESA'],
		    'cdinscesta' => $dadosEmpresaResult['CDINSCESTA'],
		    'cdpais' => $endeFiliResult['CDPAIS'],
		    'sgestado' 	=> $endeFiliResult['SGESTADO'],
			'cdmunicipio' => $endeFiliResult['CDMUNICIPIO'],
			'cdfilial' => $cdfilial,
			'cdcaixa' => $cdcaixa,
			'nrorg' => $nrorg,
			'cdloja' => $cdloja,
			'cdoperador' => $cdoperador
    	);
    }

    public function montaArrayInfoNFCE($nrseqvenda, $dtvenda, $serieNFE, $filialInfo, $connection, $VRTOTTRIBIBPT, $TOTALVENDA, $tpAmb){
		$modeloDoc      = '65';
		// 1 -> venda online, 9 -> contingencia
	    $tpEmis         = '1';
	    $cdContadorNFE  = 'NFCE_'.$filialInfo['cdfilial'].'_AMB.'.$tpEmis;
	    $nrNFE          = $this->util->geraCodigo($connection,$cdContadorNFE,$filialInfo['nrorg'], 1, 9);
	    $serieCH        = str_pad($serieNFE, 3, "0", STR_PAD_LEFT);
        $randomNrNFE    = date('d');
        $randomNrNFE   .= substr($nrseqvenda, 2);
        $fullDate       = \DateTime::createFromFormat('d/m/Y H:i:s', $dtvenda);
		$AAMM           = $fullDate->format('ym');
		$randomNrNFE    = substr($randomNrNFE, 2);
		$nrAcessoNFCE   = $filialInfo['cdestadoibge'].$AAMM.$filialInfo['nrinsjurfili'].$modeloDoc.$serieCH.$nrNFE.$tpEmis.$randomNrNFE;
		$cDV            = self::calculaDigitoVerificadorNFe($nrAcessoNFCE);
		$nrAcessoNFCE   = $nrAcessoNFCE.$cDV;

		$nfceInfo = array(
			'modeloDoc'     => $modeloDoc,
			'tpEmis'        => $tpEmis,
			'nrNFE'         => $nrNFE,
			'nrNFEInt'      => intval($nrNFE),
			'randomNrNFE'   => $randomNrNFE,
			'serieCH'       => $serieCH,
			'cDV'           => $cDV,
			'nrAcessoNFCE'  => $nrAcessoNFCE,
			'VRTOTTRIBIBPT' => $VRTOTTRIBIBPT,
			'nrseqvenda'    => $nrseqvenda,
			'totalVenda'    => $TOTALVENDA,
			'tpAmb'         => $tpAmb
		);

		return $nfceInfo;
	}

	public function getSerieNFE ($serieNFCE){
		if(!$serieNFCE['dadosSerie']['CDSERIECX']){
	    	$serieNFE = intval($serieNFCE['dadosSerie']['CDCAIXA']);
	    } else {
	    	$serieNFE = intval($serieNFCE['dadosSerie']['CDSERIECX']);
	    }
	    return $serieNFE;
	}

	public function montaArraySale($ideArray, $emitArray, $cpfDest, $totalArray, $transpArray, $infAdicArray, $paymentsArrayXml, $tpAmb) {
		$saleArray = array();
		$saleArray['ide'] = $ideArray;
		$saleArray['emit'] = $emitArray;
		if($cpfDest){
	        $destArray = self::montaArrayDest($cpfDest, $tpAmb);
	        $saleArray['dest'] = $destArray;
		}
		$indexItemAprov = 0;

		foreach ($totalArray['toSaleArray'] as $item) {
			$indexItemAprov++;
			$nItem = "nItem+".$indexItemAprov;
			$saleArray['det+'.$nItem] = $item;
		}
		unset($totalArray['toSaleArray']);
		$saleArray['total'] = $totalArray;
		$saleArray['transp'] = $transpArray;
		$idpag = 0;
		foreach($paymentsArrayXml as $paymentArray){
			$idpag++;
			$nPag = "nItem+".$idpag;
			$saleArray['pag+'.$nPag] = $paymentArray;
		}
		$saleArray['infAdic'] = $infAdicArray;
		return $saleArray;
	}

	public function montaArrayPayments($paymentModes, $filialInfo, $dtabercaix, $dtvenda, $nrseqvenda, $CDNSUHOSTTEF){
		$paymentsArrayXml = array();
		foreach ($paymentModes as $paymentMode) {
			$paymentXml = self::getPayment($paymentMode, $filialInfo['cdfilial'],$filialInfo['cdcaixa'],$dtabercaix,$dtvenda,$nrseqvenda,$filialInfo['nrorg'], $filialInfo['cdoperador'], $CDNSUHOSTTEF);
			self::mergePaymentsSEFAZ($paymentsArrayXml,$paymentXml);
		}
		return $paymentsArrayXml;
	}

	public function iniciaTransmissao($saleArray, $nfceInfo, $nfeTools, $dtvendaTZ, $tpAmb, $filialInfo, $nrseqvenda){
		try {
			$signedXMLName = self::gerarXMLValidado($saleArray, $nfeTools, $nfceInfo['nrAcessoNFCE'], false, $dtvendaTZ);
			$respostaEnvio = self::transmitirXML($nfeTools,$signedXMLName,$tpAmb);
			$resultadoTransmissao = array();
			$resultadoTransmissao['signedXMLName'] = $signedXMLName;
			if (self::validarTransmissao($respostaEnvio)) {
				$nRec     = $respostaEnvio['nRec'];
				$dhRecbto = $respostaEnvio['dhRecbto'];
				self::updateVendaEnvio($nRec,$dhRecbto,$filialInfo['cdoperador'],$filialInfo['nrorg'],$nrseqvenda,$filialInfo['cdcaixa'],$filialInfo['cdfilial']);
				$respostaRecebimento = self::consultaReciboSefaz($nRec, $tpAmb, $nfeTools);
				if(self::validarAutorizacao($respostaRecebimento)){
					$protocoloNota = $respostaRecebimento['aProt'][0];
					$idStatusNfce = self::getIdStatusNfce($protocoloNota);
					$resultadoTransmissao['idStatusNfce'] = $idStatusNfce;
					self::updateVendaProtocolo($protocoloNota,$filialInfo['cdoperador'],$filialInfo['nrorg'],$nrseqvenda,$filialInfo['cdcaixa'],$filialInfo['cdfilial']);
				}else{
					$resultadoTransmissao['idStatusNfce'] = 'T';
				}
			} else {
				$resultadoTransmissao['idStatusNfce'] = 'P';
			}
		}catch(\Exception $e){
			$resultadoTransmissao['idStatusNfce'] = 'P';
		}
		return $resultadoTransmissao;
	}

	public function transmitirNFCe($nrorg,$cdcaixa,$cdfilial,$dtabercaix,$cdloja,$dtvenda,$vrdescvenda,$cpfDest,$items,$paymentModes,$cdoperador,$idtpemisvend, $nrseqvenda, $CDNSUHOSTTEF, $VRTOTTRIBIBPT, $TOTALVENDA, $simulaImpressao) {
		$connection		= $this->entityManager->getConnection();
	    $idimpvenda 	= 'I';
	    // 1-> Producao, 2 -> Homologação 
		$tpAmb = 1;
    	$dadosEmpresaResult = self::validaDadosEmpresa($cdfilial, $nrorg);
    	if($dadosEmpresaResult['error'] == false){
   			$endeFiliResult = self::validaEnderecoFilial($cdfilial, $nrorg);
   			if($endeFiliResult['error'] == false){
			    $dadosEmitente = self::validaDadosEmitenteXML($cdfilial, $nrorg);
  				if($dadosEmitente['error'] == false){
				    $filialInfo = self::montaArrayFilial($dadosEmitente['dadosEmitente'], $dadosEmpresaResult['dadosEmpresa'], $endeFiliResult['dadosEndereco'], $nrorg, $cdcaixa, $cdfilial, $cdloja, $cdoperador);
					$emitArray = self::montaArrayEmit($filialInfo);
				    $serieNFCE = self::validaSerieNFCE($filialInfo);
  					if($serieNFCE['error'] == false){
					    $serieNFE = self::getSerieNFE($serieNFCE);
				    	$nfceInfo = self::montaArrayInfoNFCE($nrseqvenda, $dtvenda, $serieNFE, $filialInfo, $connection, $VRTOTTRIBIBPT, $TOTALVENDA, $tpAmb);
    					$fileName = $filialInfo['nrinsjurfili'].'_'.$filialInfo['nrorg'];
						$filesPath = $this->util->getConfigNFCEElement($fileName, 'pathNFeFiles');
    					if($filesPath['error'] == false){
  							self::createPath($filesPath['content']);
   							self::updateVendaDadosNFCe($filialInfo,$nrseqvenda, $nfceInfo, $tpAmb);
							$dtvendaTZ = date('c');
					        $ideArray = self::montaArrayIde($filialInfo, $nfceInfo, $serieNFE, $dtvendaTZ, $tpAmb);
							$totalArray = self::montaArrayTotal($items, $filialInfo, $nrseqvenda, $tpAmb, $vrdescvenda);
							$transpArray = self::montaArrayTransp();
							$infAdicArray = self::montaArrayInfAdic($nrseqvenda);
							$paymentsArrayXml = self::montaArrayPayments($paymentModes, $filialInfo, $dtabercaix, $dtvenda, $nrseqvenda, $CDNSUHOSTTEF);
							$saleArray = self::montaArraySale($ideArray, $emitArray, $cpfDest, $totalArray, $transpArray, $infAdicArray, $paymentsArrayXml, $tpAmb);
							$nfeTools =  self::getNfeTools($nfceInfo['modeloDoc'], $fileName);
							$transmissaoResult = self::iniciaTransmissao($saleArray, $nfceInfo, $nfeTools, $dtvendaTZ, $tpAmb, $filialInfo, $nrseqvenda);
							$nfceInfo['IDSTATUSNFCE'] = $transmissaoResult['idStatusNfce'];
							if($transmissaoResult['idStatusNfce'] != 'A'){
								$signedXMLName = self::gerarXMLValidado($saleArray, $nfeTools, $nfceInfo['nrAcessoNFCE'], true, $dtvendaTZ);
								$transmissaoResult['signedXMLName'] = $signedXMLName;
							}
							$resultadoAtualização =  self::updateVendaStatusNfce($filialInfo, $nrseqvenda, $transmissaoResult['idStatusNfce']);
							if($resultadoAtualização['error'] == false){
								$dsQrCode = self::updateVendaXML($filialInfo, $nrseqvenda, $transmissaoResult['signedXMLName']);
								if ($simulaImpressao) {
									$resultadoImpressao = array('error' => false);
								} else {
									$resultadoImpressao = $this->impressaoNFCE->imprimeDanfeNFCE($nfceInfo, $filialInfo, $items);
								}
								if($resultadoImpressao['error'] == false){
									$result = array(
										'error'=> false,
										'DSQRCODE' => $dsQrCode,
										'IDSTATUSNFCE' => $transmissaoResult['idStatusNfce'],
									);
									return $result;
								} else {
									return $resultadoImpressao;
								}
							} else {
								return $resultadoAtualização;
							}
    					} else {
							return $filesPath;
						}
  					} else {
						  return $serieNFCE;
					  }
  				} else {
					  return $dadosEmitIde;
				  }
	    	} else {
				return $endeFiliResult;
			}
		} else {
			return $dadosEmpresaResult;
		}

	}

    public static function calculaDigitoVerificadorNFe($chave){
        $nAlgarismo = 42;
        $multiplicadores = array(2,3,4,5,6,7,8,9);
        $nMultiplicador = 0;
        $soma = 0;
        $DV = 0;

        while ($nAlgarismo >= 0) {
            $algarismo = substr($chave, $nAlgarismo , 1);
            $soma += $algarismo*$multiplicadores[$nMultiplicador];
            if ($nMultiplicador == 7) {
                $nMultiplicador = 0;
            } else {
                $nMultiplicador++;
            }
            $nAlgarismo--;
		}
		
        $restoDivisao = $soma % 11;
        if ($restoDivisao > 1) {
            $DV = 11 - $restoDivisao;
		}
		
        return $DV;
    }

	public function gerarXMLValidado($saleArray, $nfeTools, $nrAcessoNFCE, $contingencia, $dtvendaTZ){
		$unsignedXML = self::gerarXML($saleArray,$nrAcessoNFCE, $contingencia, $dtvendaTZ);
		$signedXMLName = self::assinarXML($unsignedXML,$nrAcessoNFCE,$nfeTools);
   		$errosValidacao = array();
   		self::validarXML($signedXMLName,$nfeTools, $errosValidacao);
   		return $signedXMLName;
	}

	public function getQrCode($signedXMLName){
		$signedXML = self::getXMLFile($signedXMLName);
		$begin = strpos($signedXML, '<qrCode>');
		if ($begin) {
			$beginStr = substr($signedXML, $begin + 17);
			$end = strpos($beginStr, '</qrCode>');
			$qrCode = substr($beginStr, 0, $end - 3);
		} else {
			$qrCode = "QRCode inexistente.";
		}
		return $qrCode;
	}

	public function getXMLFile($XMLName){
		$XML = file_get_contents($XMLName);
		return $XML;
	}

	public function getSaleItem($cdfilial,$cdloja,$cdcaixa,$nrorg,$nrseqvenda,$nrsequitvend,$cdempresa,$cdpais,$sgestado,$cdmunicipio,$cdinscesta,$item){
		$item_array =  self::montaArray_item();
		$cdproduto = $item['CDPRODUTO'];
		$qtprodvend = $item['QTPRODVEND'];
		$vrunitvend  = $item['VRUNITVEND'];
		$idtipoitem = $item['IDTIPOITEM'];
		$idsituitem = $item['IDSITUITEM'];
		$nmproduto = $item['NMPRODUTO'];
		$vrunitvend = str_replace(',', '.', $vrunitvend);
		$qtprodvend = str_replace(',', '.', $qtprodvend);
		//---- TEMPORARIO
   		$vracritvend = isset($item['VRACRITVEND']) ? $item['VRACRITVEND'] : 0;
   		//----
		$vrdesitvend = $item['VRDESITVEND'];

		$vrdesitvend = str_replace(',', '.', $vrdesitvend);
	  	$dadosProdImpParams = self::buildParamsProdImp($nrseqvenda,$nrsequitvend, $cdfilial, $cdcaixa, $cdproduto, $nrorg);
	  	$dadosProd = $this->entityManager->getConnection()->fetchAll(self::GET_DADOSPRODIMP_XML,$dadosProdImpParams);

	  	if(count($dadosProd) != 0){
		  	$sgunidade = $dadosProd[0]['SGUNIDADE'];
		  	$cdclasfisc = $dadosProd[0]['CDCLASFISC'];
			$cdarvprod = $dadosProd[0]['CDARVPROD'];
			// --- Dados de impostos
			$cdimposto = $dadosProd[0]['CDIMPOSTO'];
			$cstImp = $dadosProd[0]['CDCSTPRODI'];
			$cstPisCof = $dadosProd[0]['CDCSTPRODPC'];
			$vraliquotaImp = $dadosProd[0]['VRPEALPRODIT'];
			$vraliquotaPis = $dadosProd[0]['VRPERPIS'];
			$vraliquotaCof = $dadosProd[0]['VRPERCOFINS'];
			$vrbasecalcicms = $dadosProd[0]['VRBASECALCICMS'];
			$cdCest = $dadosProd[0]['CDCEST'];
			$pRed = $dadosProd[0]['VRPERCREDUCAO'];

			$cfopImp = $dadosProd[0]['CDCFOPPFIS'];

		  	$vUnCom = number_format((($vrunitvend*$qtprodvend) + $vracritvend)/$qtprodvend,10, '.', '');

		  	$qCom = number_format(floatval($qtprodvend),4);

		  	$item_array['prod']['cProd'] = $cdarvprod;
		  	$item_array['prod']['cEAN'] = null;
		  	$item_array['prod']['xProd'] = preg_replace('/[^A-Za-z0-9 \-]/', '', $nmproduto);
		  	$item_array['prod']['NCM'] = $cdclasfisc;
		  	$item_array['prod']['uCom'] = $sgunidade;
		  	$item_array['prod']['qCom'] = $qtprodvend;
		  	$item_array['prod']['vUnCom'] = $vUnCom;
		  	$item_array['prod']['qCom'] = $qCom;

		  	$item_array['prod']['vProd'] = number_format(($vUnCom * $qtprodvend),2 , '.', '');
		  	$item_array['prod']['cEANTrib'] = null;
		  	$item_array['prod']['qCom'] = $qCom;
			if(floatval($vrdesitvend) > 0){
		  		$item_array['prod']['vDesc'] = number_format($vrdesitvend, 2, '.', '');
		  	}else{
		  		unset($item_array['prod']['vDesc']);
		  	}
		  	$item_array['prod']['uTrib'] = $sgunidade;
	     	$item_array['prod']['qTrib'] = $qCom;
	     	$item_array['prod']['vUnTrib'] = $vUnCom;
	     	$item_array['prod']['indTot'] = 1;//-- Compõe o total da nota


	     	if($idsituitem == 'A'){
				$baseCalcItVend = str_replace('.', ',',(floatval(str_replace(',', '.', $qtprodvend))*floatval(str_replace(',', '.', $vrunitvend))) - $vrdesitvend + $vracritvend);
				//$baseCalcItVend = ($qtprodvend*$vrunitvend) - $vrdesitvend + $vracritvend;

				$regraImpostos = array();
				//---- TRATAMENTO DOS IMPOSTOS
				if(isset($item['IMPOSTOS'])){
					//--- impostos vindos do front
				}else{
					$orig = '0';
				   	if($cdimposto != null){
					   	$nrseqitimpos      = '001';
					   	$vrimpoprodit      = $vraliquotaImp * $baseCalcItVend;
					   	$modBC             = '3';

					   	$icms_array        = self::getICMSbyCST($cstImp, $orig, $modBC, $vraliquotaImp, $vrbasecalcicms, $pRed);
					 	$pis_array 		   = self::getPISbyCST($cstPisCof, $orig, $modBC, $vraliquotaPis, $vrbasecalcicms);
					 	$cofins_array 	   = self::getCOFINSbyCST($cstPisCof, $orig, $modBC, $vraliquotaCof, $vrbasecalcicms);

					 	$tagICMS = $icms_array['TAG'];
					 	unset($icms_array['TAG']);

					 	$tagPIS = $pis_array['TAG'];
					 	unset($pis_array['TAG']);

					 	$tagCOFINS = $cofins_array['TAG'];
					 	unset($cofins_array['TAG']);

					 	$item_array['imposto']['ICMS'][$tagICMS] = $icms_array;
					 	$item_array['imposto']['PIS'][$tagPIS] = $pis_array;
					 	$item_array['imposto']['COFINS'][$tagCOFINS] = $cofins_array;

						if($cstImp == '10' || $cstImp == '30' || $cstImp =='60' || $cstImp =='70' || $cstImp=='90'){
					 		$item_array['prod']['CEST'] = $cdCest ? $cdCest : '0000000';
					 	}else{
					 		unset($item_array['prod']['CEST']);
					 	}

					 	$item_array['prod']['CFOP'] = $cfopImp;

					 	if(self::isComb($cfopImp)){
					 		$comb_array = self::montaArray_comb();
					 		if(isset($cProdANP)){
					 			$comb_array['cProdANP'] = $cProdANP;
					 			$comb_array['UFCons'] = $sgestado;
					 			$comb_array['encerrante']['nBico']   =  intval($cdbico);
	   							$comb_array['encerrante']['nBomba']  =  intval($cdbomba);
	   							$comb_array['encerrante']['nTanque'] =  intval($cdalmoxarifado);
	   							$comb_array['encerrante']['vEncIni'] =  number_format($vrencerranteinicial,3,'.','');
	   							$comb_array['encerrante']['vEncFin'] =  number_format($vrencerrantefinal,3,'.','');
	   							$item_array['prod']['comb'] = $comb_array;
	   						}
					 	}
					}
				}
			}
	  	}
		return $item_array;
	}

	const GET_DADOSPRODIMP_XML = "
       SELECT I.CDFILIAL,
          I.CDIMPOSTO, AL.CDCFOPPFIS, P.CDCEST, AL.VRPERCREDUCAO,
          I.CDCAIXA, I.NRSEQVENDA, I.NRSEQUITVEND, I.VRPEALPRODIT, I.VRPERPIS,
          I.VRPERCOFINS, I.CDCSTPRODI, I.CDCSTPRODPC, I.VRBASECALCICMS, P.CDARVPROD,
          P.NMPRODUTO, P.CDCLASFISC, P.SGUNIDADE, I.VRTOTTRIBIBPT
        FROM ITVENDAIMPOS I
	        JOIN PRODUTO P
	        ON P.CDPRODUTO = :CDPRODUTO
	        AND P.NRORG    = I.NRORG
	        AND P.IDATIVO  = 'S'
        JOIN ALIQIMPFIS AL
	        ON AL.CDPRODUTO    = P.CDPRODUTO
	        AND AL.CDFILIAL    = I.CDFILIAL
	        AND AL.NRORG       = I.NRORG
	        AND AL.IDATIVO     = 'S'
        WHERE I.CDFILIAL   = :CDFILIAL
          AND I.CDCAIXA      = :CDCAIXA
          AND I.NRSEQVENDA   = :NRSEQVENDA
          AND I.NRSEQUITVEND = :NRSEQUITVEND
          AND I.NRORG        = :NRORG
          AND I.IDATIVO      = 'S'
    ";

	public function buildParamsProdImp($nrseqvenda,$nrsequitvend, $cdfilial, $cdcaixa, $cdproduto, $nrorg){
		$prodImpParams = array(
			'NRSEQVENDA'   => $nrseqvenda,
			'NRSEQUITVEND' => $nrsequitvend,
			'CDFILIAL'     => $cdfilial,
			'CDCAIXA'      => $cdcaixa,
			'CDPRODUTO'    => $cdproduto,
			'NRORG'        => $nrorg
   		);
   		return $prodImpParams;
	}

	public function getPayment($payment, $cdfilial,$cdcaixa,$dtabercaix,$dtvenda,$nrseqvenda,$nrorg, $cdoperador, $CDNSUHOSTTEF){
		//--- Payment properties
		$idtipomovive  = self::ENTRADA;
		$vrmovivend    = str_replace(',','.', $payment['VALOR']);
		$cdtiporece    = $payment['CDTIPORECE'];
        $payment_array = self::montaArray_pag();
		$idtiporeceParams = array(
								'CDTIPORECE' => $cdtiporece,
								'NRORG' => $nrorg
							);
		$idtiporeceQuery = $this->entityManager->getConnection()->fetchAll(self::GET_IDTIPORECE, $idtiporeceParams);
		$idtiporeceSAAS = $idtiporeceQuery[0]['IDTIPORECE'];
		$idtiporeceSEFAZ = self::getSEFAZIdtiporece($idtiporeceSAAS);
		$payment_array['tPag'] = $idtiporeceSEFAZ;
		$payment_array['vPag'] = number_format($vrmovivend,2, '.', '');
		if($idtiporeceSEFAZ == '03' || $idtiporeceSEFAZ == '04'){
			$card_array = $this->montaArray_card();
			$card_array['tpIntegra'] = 1;
			$card_array['CNPJ'] = $idtiporeceQuery[0]['CDADMINCART'];
			$card_array['tBand'] = $idtiporeceQuery[0]['CDBANCARTCR'];
			$card_array['cAut'] = $CDNSUHOSTTEF;
            $payment_array['card'] = $card_array;
        }
		return $payment_array;
	}

	 const GET_IDTIPORECE = "
        SELECT CDTIPORECE, NMTIPORECE, IDTIPORECE, CDADMINCART, CDBANCARTCR
          FROM TIPORECE
         WHERE CDTIPORECE = :CDTIPORECE
           AND NRORG      = :NRORG
    ";

   	public function montaArrayIde($filialInfo,$nfceInfo, $serieNFE, $dtvendaTZ ,$tpAmb){
   		// --- IDENTIFICAÇÃO DA NOTA
		$ide =  array(
					 'cUF'    	=>  $filialInfo['cdestadoibge'], // CDESTADOIBGE
			         'cNF'    	=>  substr($nfceInfo['randomNrNFE'], 2), // NUMERO ALEATÓRIO - 8 DIG
			         'natOp'  	=>  'Descrição da natureza de operação : Venda', // DESCRIÇÃO DA NATUREZA DE OPERAÇÃO
			         'indPag' 	=>  2, // PAGAMENTO : 0 - A VISTA, 1 - PRAZO , 2 - OUTROS
			         'mod'   	=>  $nfceInfo['modeloDoc'], // MODELO : 55 - NFe , 65 - NFCe
			         'serie'  	=>  $serieNFE, // CDSERIENFE
			         'nNF'	 	=>  $nfceInfo['nrNFEInt'], // NÚMERO DA NOTA (NOVOCODIGO NFCE_FILIAL_AMBIENTE)
			         'dhEmi'  	=>  $dtvendaTZ, // FULL DATETIME, MOMENTO DE EMISSÃO
			         'tpNF'   	=>  1, // TIPO DA NOTA FISCAL : 0 - ENTRADA, 1 - SAIDA
			         'idDest' 	=>  1, // ID DESTINO: 1 - INTERNO, 2 - INTERESTADUAL , 3 - EXTERIOR
			         'cMunFG' 	=>  $filialInfo['cdmunicibge'], // CDMUNICIBGE
			         'tpImp'  	=>	4, // TIPO DE IMPRESSAO : 4 - DANFE NFCe
			         'tpEmis' 	=>  $nfceInfo['tpEmis'], // TIPO DE EMISSÃO : 1 - NORMAL
			         'cDV'    	=>  $nfceInfo['cDV'], // DÍGITO VERIFICADOR nrAcessoNFCE
			         'tpAmb'  	=>  $tpAmb, // TIPO DE AMBIENTE : 1 - PRODUÇÃO, 2 - HOMOLOGAÇÃO
			         'finNFe' 	=>  1, // FINALIDADE DA NOTA : 1 - NORMAL
			         'indFinal' =>	1, // CONSUMIDOR FINAL : 1 - SIM
			         'indPres'  =>	1, // OPERACAO PRESENCIAL : 1 - SIM
			         'procEmi'  =>	0, // PROCESSO DE EMISSÃO : 0 - POR APP DO CONTRIBUINTE
			         'verProc'	=>	1  // VERSÃO DO PROCESSO EMISSOR
	    );
		return $ide;
   	}

   	public function montaArrayEmit($filialInfo){
   		// --- IDENTIFICAÇÃO DO EMITENTE
		// - identificação do endereço do emitente
		$enderEmit = array(
						'xLgr' 	  =>	$filialInfo['dsendefili'], // DSENDEFILI
						'nro'	  =>	$filialInfo['nrendefilisep'], // NRENDEFILISEP
						'xBairro' =>	$filialInfo['nmbairfili'], // NMBAIRFILI
						'cMun'	  =>	$filialInfo['cdmunicibge'], // CDMUNICIBGE
						'xMun'	  =>	$filialInfo['nmmunicipio'], // NMMUNICIPIO
						'UF'	  =>	$filialInfo['sgestadofili'], // SGESTADO
						'CEP'	  =>	$filialInfo['nrcepfili'], // CEP
						'cPais'	  =>	$filialInfo['cdpaisbacen'], // CDPAISBACEN
						'xPais'	  =>	$filialInfo['nmpais']  // NMPAIS
					 );
		// - identificação do emitente
		$emit = array(
					'CNPJ'		 =>	$filialInfo['nrinsjurfili'], // NRINSJURFILI
					'xNome'	     =>	$filialInfo['nmrazsocfili'], // NMRAZSOCFILI
					'xFant'	     =>	$filialInfo['nmfilial'], // NMFILIAL
					'enderEmit'  => $enderEmit, // ENDERECO EMITENTE
					'IE'		 => $filialInfo['cdinscestafili'], // CDINSCESTA
					'IM'		 => $filialInfo['cdinscmunifili'], // CDINSCMUNI
					'CNAE'		 => $filialInfo['cdcnae'], // CDCNAE
					'CRT'		 => $filialInfo['cdsitucrt']  // CDSITUCRT
				);

		return $emit;
   	}

   	public function montaArrayDest($cpfDest, $tpAmb){
   		// --- IDENTIFICAÇÃO DO DESTINATARIO
		$dest_array = array(
			'CPF'		 =>	preg_replace('/[^0-9]/', '', $cpfDest),
			'xNome'      => null,
			'indIEDest'	 =>	'9'
		);
   		if($tpAmb == 2)
        	$dest_array['xNome'] = "NF-E EMITIDA EM AMBIENTE DE HOMOLOGACAO - SEM VALOR FISCAL";
        else
        	unset($dest_array['xNome']);
		$dest_array['dest'] = $dest_array;
		return $dest_array;
   	}

   	public function montaArray_item(){
   		// --- IDENTIFICAÇÃO DE UM ITEM DA VENDA
		// - identificação do produto que compõe o item
		$prod = array(
					'cProd'		 => null, // CDARVPROD
					'cEAN'		 => NULL, // ?????????
					'xProd'		 => null, // NMPRODUTO
					'NCM'		 => null, // CDCLASFISC
					'CEST'       => null, // CDCEST
					'CFOP'		 => null, // Consulta regra fiscal
					'uCom'		 => null, // SGUNIDADE
					'qCom'		 => null, // QTPRODVEND - 4 casas decimais
					'vUnCom'	 => null, // VRUNITPROD - 10 casas decimais
					'vProd'		 => null, // Valor Bruto
					'cEANTrib'	 => null, // ?????????
					'uTrib'		 => null, // SGUNIDADE
					'qTrib'		 => null, // qCom
					'vUnTrib'	 => null, // vUnCom
					'vDesc'	     => null, // VRDESITVEND
					'indTot'	 => null  // COMPÕE O TOTAL DA NOTA: 0 - NÃO; 1 - SIM
			 	);

		$imposto = array(
			'ICMS'    => array(),
			'PIS'     => array(),
			'COFINS'  => array()
		);

		// - identificação do imposto de um item
		$item = array(
				'prod' 	   =>	$prod,
				'imposto'  =>   $imposto
		);

		return $item;
   	}

   	public function montaArray_comb(){
   		$encerrante  = array(
   			'nBico'   =>  null,
   			'nBomba'  =>  null,
   			'nTanque' =>  null,
   			'vEncIni' =>  null,
   			'vEncFin' =>  null
   		);
   		$comb = array(
   			'cProdANP'	=> null,
   			'UFCons'	=> null,
   			'encerrante' => $encerrante
   		);
   	}

   	public function montaArray_pag(){
   		// --- IDENTIFICAÇÃO DE UM PAGAMENTO DA VENDA
		$pag = array(
					'tPag'		 => null, // TIPO DE PAGAMENTO - idtiporeceSEFAZ
					'vPag'		 => null, // VALOR DO PAGAMENTO - VRMOVIVEND
	     	 	);
		return $pag;
   	}

   	public function montaArray_card(){
   		$card = array(
   					'tpIntegra'  => null, //tipo de integração
					'CNPJ' => null,
					'tBand' => null,
					'cAut' => null
   				);

   		return $card;
   	}

   	public function montaArray_ICMS00(){
   		$icms00 = array(
   			'orig' => null,
   			'CST' => '00',
   			'modBC' => null,
   			'vBC' => null,
   			'pICMS' => null,
   			'vICMS' => null
   		);
   		return $icms00;
   	}

   	public function montaArray_ICMS10(){
   		$icms10 = array(
   			'orig' => null,
   			'CST' => '10',
   			'modBC' => null,
   			'vBC' => null,
   			'pICMS' => null,
   			'vICMS' => null
   		);
   		return $icms10;
   	}

   	public function montaArray_ICMS20(){
   		$icms20 = array(
   			'orig' => null,
   			'CST' => '20',
   			'modBC' => null,
   			'pRedBC' => null,
   			'vBC' => null,
   			'pICMS' => null,
   			'vICMS' => null
   		);
   		return $icms20;
   	}

   	public function montaArray_ICMS30(){

   		$icms30 = array(
   			'orig' => null,
   			'CST' => '30'
   		);

   		return $icms30;
   	}

   	public function montaArray_ICMS40(){

   		$icms40 = array(
   			'orig' => null,
   			'CST' => '40'
   		);

   		return $icms40;
   	}

   	public function montaArray_ICMS41(){

   		$icms41 = array(
   			'orig' => null,
   			'CST' => '41'
   		);

   		return $icms41;
   	}

   	public function montaArray_ICMS50(){

   		$icms50 = array(
   			'orig' => null,
   			'CST' => '50'
   		);

   		return $icms50;
   	}


   	public function montaArray_ICMS51(){

   		$icms51 = array(
   			'orig' => null,
   			'CST' => '51',
   			'modBC' => null,
   			'pRedBC' => null,
   			'vBC' => null,
   			'pICMS' => null,
   			'vICMS' => null
   		);
   		return $icms51;
   	}

   	public function montaArray_ICMS60(){

   		$icms60 = array(
   			'orig' => null,
   			'CST' => '60'
   		);

   		return $icms60;
   	}

   	public function montaArray_ICMS70(){

   		$icms70 = array(
   			'orig'    => null,
   			'CST'     => '70',
   			'modBC'   => null,
   			'pRedBC'  => null,
   			'vBC'     => null,
   			'pICMS'   => null,
   			'vICMS'   => null,
   			'modBCST' => '3',
            'pMVAST'  => '0.00',
            'pRedBCST'=> '0.00',
            'vBCST'   => '0.00',
            'pICMSST' => '0.00',
            'vICMSST' => '0.00'
   		);
   		return $icms70;
   	}

   	public function montaArray_ICMS90(){

   		$icms90 = array(
   			'orig'    => null,
   			'CST'     => '90',
   			'modBC'   => null,
   			'vBC'     => null,
   			'pRedBC'  => null,
   			'pICMS'   => null,
   			'vICMS'   => null
   		);
   		return $icms90;
   	}

   	public function montaArray_PISAliq(){

   		$pisAliq = array(
   			'CST'  => null,
   			'vBC'  => null,
			'pPIS' => null,
			'vPIS' => null
   		);

   		return $pisAliq;
   	}

   	public function montaArray_PISNT(){

   		$pisNT = array(
   			'CST'  => null
   		);

   		return $pisNT;
   	}

   	public function montaArray_PISQtde(){

   		$pisQtde = array(
   			'CST'  => null,
   			'vBC'  => null,
			'pPIS' => null,
			'vPIS' => null
   		);

   		return $pisQtde;
   	}

   	public function montaArray_PISOutr(){

   		$pisOutr = array(
   			'CST'  	  => null,
   			'vBC'  	  => null,
			'pPIS' => null,
			'vPIS' => null
   		);

   		return $pisOutr;
   	}

   	public function montaArray_COFINSAliq(){

   		$cofinsAliq = array(
   			'CST'  	  => null,
   			'vBC'  	  => null,
			'pCOFINS' => null,
			'vCOFINS' => null
   		);

   		return $cofinsAliq;
   	}

   	public function montaArray_COFINSQtde(){

   		$cofinsQtde = array(
   			'CST'  	  => null,
   			'vBC'  	  => null,
			'pCOFINS' => null,
			'vCOFINS' => null
   		);

   		return $cofinsQtde;
   	}

   	public function montaArray_COFINSNT(){

   		$cofinsNT = array(
   			'CST'  	  => null
   		);

   		return $cofinsNT;
   	}

   	public function montaArray_COFINSOutr(){

   		$cofinsOutr = array(
   			'CST'  	  => null,
   			'vBC'  	  => null,
			'pCOFINS' => null,
			'vCOFINS' => null
   		);

   		return $cofinsOutr;
   	}


   	public function montaArrayTotal($items, $filialInfo, $nrseqvenda, $tpAmb, $vrdescvenda){
   		$icmsArray = self::montaArrayICMS();
		$totalArray = array();
		$totalArray['ICMSTot'] = $icmsArray;
	    $indexItemAprov = 0;
		foreach($items as $item){
			if($item['IDSITUITEM'] == "A"){
				$indexItemAprov++;
				$nItem = "nItem+".$indexItemAprov;
				$nrsequitvend = str_pad($indexItemAprov ,3,'0',STR_PAD_LEFT);
				$itemXml = self::getSaleItem($filialInfo['cdfilial'],$filialInfo['cdloja'],$filialInfo['cdcaixa'],$filialInfo['nrorg'],$nrseqvenda,$nrsequitvend,$filialInfo['cdempresa'],$filialInfo['cdpais'],$filialInfo['sgestado'],$filialInfo['cdmunicipio'],$filialInfo['cdinscesta'],$item);
				if($tpAmb == 2 && $indexItemAprov == 1){
					$itemXml['prod']['xProd'] = 'NOTA FISCAL EMITIDA EM AMBIENTE DE HOMOLOGACAO - SEM VALOR FISCAL';
				}
				// $saleArray['det+'.$nItem] = $itemXml;
				$totalArray['toSaleArray']['det+'.$nItem] = $itemXml;
				if($itemXml['prod']['indTot'] == '1'){
					$icms = $itemXml['imposto']['ICMS'];
					$totalArray = self::updateTotal($itemXml,$totalArray, $icms);
				}
			}
		}
		$totalArray['ICMSTot']['vDesc'] = number_format($totalArray['ICMSTot']['vDesc'] + $vrdescvenda,2);
		$totalArray['ICMSTot']['vNF'] = self::calcTotalNF($totalArray);
	    return $totalArray;
   	}

   	public function calcTotalNF($totalArray){
   		return number_format($totalArray['ICMSTot']['vProd'] - $totalArray['ICMSTot']['vDesc'] + $totalArray['ICMSTot']['vST'] + $totalArray['ICMSTot']['vFrete'] + $totalArray['ICMSTot']['vSeg'] + $totalArray['ICMSTot']['vOutro'] + $totalArray['ICMSTot']['vII'] + $totalArray['ICMSTot']['vIPI'] - $totalArray['ICMSTot']['vICMSDeson'],2, '.', '');
   	}

   	public function montaArrayICMS(){
   		$icmsArray = array(
         	'vBC'		 =>	0,
         	'vICMS' 	 =>	0,
         	'vICMSDeson' =>	0,
         	'vBCST' 	 =>	0,
         	'vST' 		 =>	0,
    		'vProd'      =>	0,
    		'vFrete'	 =>	0,
    		'vSeg'		 =>	0,
    		'vDesc'		 =>	0,
    		'vII'		 =>	0,
    		'vIPI'		 =>	0,
    		'vPIS' 		 =>	0,
    		'vCOFINS'	 =>	0,
    		'vOutro'	 =>	0,
    		'vNF'		 =>	0
	    );
	    return $icmsArray;
   	}

   	public function montaArrayTransp(){
   		$transp = array(
         	'modFrete'	 =>	9
        );
        return $transp;
   	}

   	public function montaArrayInfAdic($nrseqvenda){
   		$infAdic = array(
         	'obsCont+xCampo+NrSeqVen-IdOrigemVen' => array(
         		'xTexto' => $nrseqvenda.'-BAL_FNC'
         	)
        );
        return $infAdic;
   	}

   	public function getSEFAZIdtiporece($idtiporeceSAAS){
   		$idtiporeceSEFAZ = '99';
   		switch($idtiporeceSAAS){
   			case '1':
   				$idtiporeceSEFAZ = '03';
   				break;
   			case '2':
   				$idtiporeceSEFAZ = '04';
   				break;
   			case '3':
   				$idtiporeceSEFAZ = '02';
   				break;
   			case '4':
   				$idtiporeceSEFAZ = '01';
   				break;
   			case '5':
   				$idtiporeceSEFAZ = '11';
   				break;
   			case 'A':
   				$idtiporeceSEFAZ = '99';
   				break;
   			case '9':
   			case '8':
   				$idtiporeceSEFAZ = '05';
   				break;
   			default:
   				break;
	   	}
	   	return $idtiporeceSEFAZ;
	}
    public function mergePaymentsSEFAZ(&$payments, $newPayment){
    	$exists = false;
		foreach($payments as $key=>$payment){
    		if($payment['tPag'] == $newPayment['tPag']){
	   			$payment['vPag'] = number_format($payment['vPag'] + $newPayment['vPag'],2, '.', '');
	   			$payments[$key] = $payment;
	   			$exists = true;
	   			break;
	   		}
	   	}
	   	if(!$exists){
	   		$paymentsSize = count($payments);
	   		$payments[$paymentsSize] = $newPayment;
	   	}
	}

	public function getICMSbyCST($cstImp, $orig, $modBC, $vraliquotaImp, $vrbasecalcicms, $pRed){
		$icms_array = array();
		$tagXML = '';
		$vraliquotaImp = $vraliquotaImp/100;
		if($cstImp == '00'){
			$tagXML = 'ICMS00';
			$icms_array = self::montaArray_ICMS00();
			$icms_array['orig']  = 	$orig;
			$icms_array['modBC'] = 	$modBC;
			$icms_array['vBC']   = 	number_format(str_replace(',', '.',$vrbasecalcicms),2, '.', '');
			$icms_array['pICMS'] =  number_format($vraliquotaImp*100,2);

			$vICMS = number_format(str_replace(',', '.',$vraliquotaImp),3) * number_format(str_replace(',', '.',$vrbasecalcicms),2);

			$vICMS = number_format($vICMS, 2);
			$icms_array['vICMS'] = $vICMS;
		}else if($cstImp == '10'){
			$tagXML = 'ICMS10';
			$icms_array = self::montaArray_ICMS10();
			$icms_array['orig']  = 	$orig;
			$icms_array['modBC'] = 	$modBC;
			$icms_array['vBC']   = 	number_format(str_replace(',', '.',$vrbasecalcicms),2, '.', '');
			$icms_array['pICMS'] =  number_format($vraliquotaImp*100,2);
			$vICMSStr = (string)(number_format(str_replace(',', '.',$vraliquotaImp),2) * number_format(str_replace(',', '.',$vrbasecalcicms),2));
			$dotPos = strpos($vICMSStr,'.');
			$vICMS = substr($vICMSStr,0,$dotPos+3);
			$icms_array['vICMS'] = $vICMS;
		}else if($cstImp == '20'){
			$tagXML = 'ICMS20';
			$icms_array = self::montaArray_ICMS20();
			$icms_array['orig']  = $orig;
			$icms_array['modBC'] = 	$modBC;
			$icms_array['pRedBC'] = number_format(str_replace(',', '.',$pRed),2, '.', '');
			$icms_array['vBC']   = 	number_format(str_replace(',', '.',$vrbasecalcicms),2, '.', '');
			$icms_array['pICMS'] =  number_format($vraliquotaImp*100,2);
			$vICMSStr = (string)(number_format(str_replace(',', '.',$vraliquotaImp),2) * number_format(str_replace(',', '.',$vrbasecalcicms),2));
			$dotPos = strpos($vICMSStr,'.');
			$vICMS = substr($vICMSStr,0,$dotPos+3);
			$icms_array['vICMS'] = $vICMS;
		}else if($cstImp == '30'){
			$tagXML = 'ICMS30';
			$icms_array = self::montaArray_ICMS30();
			$icms_array['orig']  = $orig;
		}else if($cstImp == '40'){
			$tagXML = 'ICMS40';
			$icms_array = self::montaArray_ICMS40();
			$icms_array['orig']  = $orig;
		}else if($cstImp == '41'){
			$tagXML = 'ICMS41';
			$icms_array = self::montaArray_ICMS41();
			$icms_array['orig']  = $orig;
		}else if($cstImp == '50'){
			$tagXML = 'ICMS50';
			$icms_array = self::montaArray_ICMS50();
			$icms_array['orig']  = $orig;
		}else if($cstImp == '51'){
			$tagXML = 'ICMS51';
			$icms_array = self::montaArray_ICMS51();
			$icms_array['orig']  = $orig;
			$icms_array['modBC'] = 	$modBC;
			$icms_array['pRedBC'] = number_format(str_replace(',', '.',$pRed),2, '.', '');
			$icms_array['vBC']   = 	number_format(str_replace(',', '.',$vrbasecalcicms),2, '.', '');
			$icms_array['pICMS'] =  number_format($vraliquotaImp*100,2);
			$vICMSStr = (string)(number_format(str_replace(',', '.',$vraliquotaImp),2) * number_format(str_replace(',', '.',$vrbasecalcicms),2));
			$dotPos = strpos($vICMSStr,'.');
			$vICMS = substr($vICMSStr,0,$dotPos+3);
		}else if($cstImp == '60'){
			$tagXML = 'ICMS60';
			$icms_array = self::montaArray_ICMS60();
			$icms_array['orig']  = $orig;
		}else if($cstImp == '70'){
			$tagXML = 'ICMS70';
			$icms_array = self::montaArray_ICMS70();
			$icms_array['orig']  = $orig;
			$icms_array['modBC'] = 	$modBC;
			$icms_array['pRedBC'] = number_format(str_replace(',', '.',$pRed),2, '.', '');
			$icms_array['vBC']   = 	number_format(str_replace(',', '.',$vrbasecalcicms),2, '.', '');
			$icms_array['pICMS'] =  number_format($vraliquotaImp*100,2);
			$vICMSStr = (string)(number_format(str_replace(',', '.',$vraliquotaImp),2) * number_format(str_replace(',', '.',$vrbasecalcicms),2));
			$dotPos = strpos($vICMSStr,'.');
			$vICMS = substr($vICMSStr,0,$dotPos+3);
			$icms_array['vICMS'] = $vICMS;
		}else if($cstImp == '90'){
			$tagXML = 'ICMS90';
			$icms_array = self::montaArray_ICMS90();
			$icms_array['orig']  = $orig;
			$icms_array['modBC'] = 	$modBC;
			$icms_array['vBC']   = 	number_format(str_replace(',', '.',$vrbasecalcicms),2, '.', '');
			$icms_array['pRedBC'] = number_format(str_replace(',', '.',$pRed),2, '.', '');
			$icms_array['pICMS'] =  number_format($vraliquotaImp*100,2);
			$vICMSStr = (string)(number_format(str_replace(',', '.',$vraliquotaImp),2) * number_format(str_replace(',', '.',$vrbasecalcicms),2));
			$dotPos = strpos($vICMSStr,'.');
			$vICMS = substr($vICMSStr,0,$dotPos+3);
			$icms_array['vICMS'] = $vICMS;
		}
		$icms_array['TAG'] = $tagXML;
		return $icms_array;
	}

	public function getPISbyCST($cstPis, $orig, $modBC, $vraliquotaPis, $vrbasecalcicms){
		$pis_array = array();
		$vraliquotaPis  = str_replace(',','.',$vraliquotaPis);
		$vraliquotaPis = $vraliquotaPis/100;
		$vrbasecalcicms = str_replace(',','.',$vrbasecalcicms);
		$tagXML = '';
		if($cstPis == '01' || $cstPis == '02'){
			$pis_array = self::montaArray_PISAliq();
			$tagXML = 'PISAliq';
			$pis_array['pPIS'] = number_format($vraliquotaPis*100,2);
			$pis_array['vBC']  = number_format($vrbasecalcicms,2, '.', '');
			$pis_array['vPIS'] = number_format($vrbasecalcicms * $vraliquotaPis,2);
		}else if($cstPis == '03'){
			$tagXML = 'PISQtde';
			$pis_array = self::montaArray_PISQtde();
			$pis_array['pPIS'] = number_format($vraliquotaPis*100,2);
			$pis_array['vBC']  = number_format($vrbasecalcicms,2, '.', '');
			$pis_array['vPIS'] = number_format($vrbasecalcicms * $vraliquotaPis,2);
		}else if($cstPis == '04' || $cstPis == '05' || $cstPis == '06' || $cstPis == '07' || $cstPis == '08' || $cstPis == '09'){
			$tagXML = 'PISNT';
			$pis_array = self::montaArray_PISNT();
		}else if($cstPis == '49' || $cstPis == '50' || $cstPis == '51' ||
			     $cstPis == '52' || $cstPis == '53' || $cstPis == '54' || $cstPis == '55' ||
			     $cstPis == '56' || $cstPis == '60' || $cstPis == '61' || $cstPis == '62' ||
			     $cstPis == '63' || $cstPis == '64' || $cstPis == '65' || $cstPis == '66' ||
			     $cstPis == '67' || $cstPis == '70' || $cstPis == '71' || $cstPis == '72' ||
			     $cstPis == '73' || $cstPis == '74' || $cstPis == '75' || $cstPis == '98' ||
			     $cstPis == '99'){
			$tagXML = 'PISOutr';
			$pis_array = self::montaArray_PISOutr();
			$pis_array['pPIS'] = number_format($vraliquotaPis*100,2);
			$pis_array['vBC']  = number_format($vrbasecalcicms,2, '.', '');
			$pis_array['vPIS'] = number_format($vrbasecalcicms * $vraliquotaPis,2);
		}
		$pis_array['CST'] = $cstPis;
		$pis_array['TAG'] = $tagXML;
		return $pis_array;
	}

	public function getCOFINSbyCST($cstCof, $orig, $modBC, $vraliquotaCof, $vrbasecalcicms){
		$cof_array = array();
		$tagXML = '';
		$vraliquotaCof  = str_replace(',','.',$vraliquotaCof);
		$vrbasecalcicms = str_replace(',','.',$vrbasecalcicms);
		$vraliquotaCof = $vraliquotaCof/100;
		if($cstCof == '01' || $cstCof == '02'){
			$tagXML = 'COFINSAliq';
			$cof_array = self::montaArray_COFINSAliq();
			$cof_array['pCOFINS'] = number_format($vraliquotaCof*100,2);
			$cof_array['vBC']     = number_format($vrbasecalcicms,2, '.', '');
			$cof_array['vCOFINS'] = number_format($vrbasecalcicms * $vraliquotaCof,2);
		}else if($cstCof == '03'){
			$tagXML = 'COFINSQtde';
			$cof_array = self::montaArray_COFINSQtde();
			$cof_array['pCOFINS'] = number_format($vraliquotaCof*100,2);
			$cof_array['vBC']     = number_format($vrbasecalcicms,2, '.', '');
			$cof_array['vCOFINS'] = number_format($vrbasecalcicms * $vraliquotaCof,2);
		}else if($cstCof == '04'  || $cstCof == '05' || $cstCof == '06' || $cstCof == '07' || $cstCof == '08' || $cstCof == '09'){
			$tagXML = 'COFINSNT';
			$cof_array = self::montaArray_COFINSNT();
		}else if($cstCof == '49' || $cstCof == '50' || $cstCof == '51' ||
			     $cstCof == '52' || $cstCof == '53' || $cstCof == '54' || $cstCof == '55' ||
			     $cstCof == '56' || $cstCof == '60' || $cstCof == '61' || $cstCof == '62' ||
			     $cstCof == '63' || $cstCof == '64' || $cstCof == '65' || $cstCof == '66' ||
			     $cstCof == '67' || $cstCof == '70' || $cstCof == '71' || $cstCof == '72' ||
			     $cstCof == '73' || $cstCof == '74' || $cstCof == '75' || $cstCof == '98' ||
			     $cstCof == '99'){
			$tagXML = 'COFINSOutr';
			$cof_array = self::montaArray_COFINSOutr();
			$cof_array['pCOFINS'] = number_format($vraliquotaCof*100,2);
			$cof_array['vBC']     = number_format($vrbasecalcicms,2, '.', '');
			$cof_array['vCOFINS'] = number_format($vrbasecalcicms * $vraliquotaCof,2);
		}

		$cof_array['CST'] = $cstCof;
		$cof_array['TAG'] = $tagXML;

		return $cof_array;
	}

	public function gerarXML($saleArray, $nrAcessoNFCE, $contingencia, $dtvendaTZ){
		$dom = self::generatesStringXMLByIndexedArray($saleArray,$nrAcessoNFCE, $contingencia, $dtvendaTZ);
	    $XML = self::formataCaracteresXML($dom->saveXML());
		$XMLName = ENTRADAS_XML_PATH."NFCE".$nrAcessoNFCE.".xml";
		$handle = fopen($XMLName, "w");
		fwrite($handle, $XML);
		fclose($handle);
		return $XML;
	}

	public function createPath($filesPath){
		$this->util->createFolder($filesPath);
		FilesFolders::createFolders($filesPath);
		$entradasPath = $filesPath.DIRECTORY_SEPARATOR."homologacao".DIRECTORY_SEPARATOR."entradas".DIRECTORY_SEPARATOR;
		$assinadasPath = $filesPath.DIRECTORY_SEPARATOR."homologacao".DIRECTORY_SEPARATOR."assinadas".DIRECTORY_SEPARATOR;
		if (!defined('ENTRADAS_XML_PATH')) {
			define('ENTRADAS_XML_PATH', $entradasPath);
		}
		if (!defined('ASSINADAS_XML_PATH')) {
			define('ASSINADAS_XML_PATH', $assinadasPath);
		}
	}

	public function getNfeTools($modeloDoc, $fileName){
		$NFCEpath = $this->util->getXMLParameter('PATH_NFCE_SAT');
		$NFCEpath = $NFCEpath.DIRECTORY_SEPARATOR."NFCE".DIRECTORY_SEPARATOR."config".DIRECTORY_SEPARATOR;
		$filePath = $NFCEpath.$fileName.'.json';

		$nfeTools = new ToolsNFe($filePath);
		$nfeTools->setModelo($modeloDoc);
		return $nfeTools;
	}

	public function assinarXML($unsignedXML,$nrAcessoNFCE,$nfeTools){
		$signedXMLName = ASSINADAS_XML_PATH."NFCE".$nrAcessoNFCE.".xml";
		$signedXML = $nfeTools->assina($unsignedXML);
		$sHandle = fopen($signedXMLName, "w");
		fwrite($sHandle, $signedXML);
		fclose($sHandle);
		return $signedXMLName;
	}

	public function validarXML($signedXMLName, $nfeTools, &$erros = array()){
		$XMLValido = false;
     	if($nfeTools->validarXml($signedXMLName)){
			$XMLValido = true;
		}else{
			$erros = $nfeTools->errors;
		}
		return $XMLValido;
	}


	public function transmitirXML($nfeTools,$signedXMLName,$tpAmb){
		$signedXML = file_get_contents($signedXMLName);
		$aRespostaEnvio = array();
		$idLote = '';
		$indSinc = '0';
		$flagZip = false;
		$nfeTools->sefazEnviaLote($signedXML, $tpAmb, $idLote, $aRespostaEnvio, $indSinc, $flagZip);
		return $aRespostaEnvio;
	}

	public function validarTransmissao($respostaEnvio){
		$transmitido = false;
		if(isset($respostaEnvio['bStat'])){
			$transmitido = $respostaEnvio['bStat'] && $respostaEnvio['cStat'] == '103';
		}
		return $transmitido;
	}

	public function validarAutorizacao($respostaRecebimento){
		$autorizada = false;
		if(isset($respostaRecebimento['bStat'])){
			$autorizada = $respostaRecebimento['bStat'] && $respostaRecebimento['cStat'] == '104';
		}
		return $autorizada;
	}

	public function consultaReciboSefaz($nRec, $tpAmb, $nfeTools){
		$respostaRecebimento = array();
		$nfeTools->sefazConsultaRecibo($nRec,$tpAmb,$respostaRecebimento);
		return $respostaRecebimento;
	}

	public function generatesStringXMLByIndexedArray($saleArray, $nrAcessoNFCE, $contingencia, $dtvendaTZ){
		$dom = new \DOMDocument('1.0', 'utf-8');
		//------ Atributos
		$xmlns = $dom->createAttribute('xmlns');
		$xmlns->value = 'http://www.portalfiscal.inf.br/nfe';

		//------ NFe
		$NFe = $dom->createElement('NFe');
		$NFe->appendChild($xmlns);
		$dom->appendChild($NFe);

		//------ infNFe
		$infNFe = $dom->createElement('infNFe');
		$Id = $dom->createAttribute('Id');
		$infNFeVersao = $dom->createAttribute('versao');
		$Id->value = 'NFe'.$nrAcessoNFCE;
		$infNFeVersao->value = '3.10';

		$infNFe->appendChild($Id);
		$infNFe->appendChild($infNFeVersao);

		$NFe->appendChild($infNFe);

		if($contingencia){
			$saleArray['ide']['dhCont'] = $dtvendaTZ;
			$saleArray['ide']['xJust'] = 'Motivo Emissao em Contingencia: Caixa OffLine';
			$saleArray['infAdic']['infCpl'] = 'NFCe EMITIDA EM CONTINGENCIA';
			ksort($saleArray['infAdic']);
		}
     	self::arrayElementsToXML($saleArray,$infNFe,$dom);

     	return $dom;

	}


	public function arrayElementsToXML($array,&$currentTag,&$dom){
		foreach($array as $key=>$value){
			$newElement =  null;
			$keyAsArray = explode('+',$key);
			if(count($keyAsArray) > 1){
				$key = $keyAsArray[0];
				$newElement = $dom->createElement($key);
				if($key == 'det' || $key == 'obsCont'){
					$attrName = $keyAsArray[1];
					$attrValue = $keyAsArray[2];
					$attribute = $dom->createAttribute($attrName);
					$attribute->value = $attrValue;
					$newElement->appendChild($attribute);
				}
				$currentTag->appendChild($newElement);
				if(is_array($value)){
					self::arrayElementsToXML($value,$newElement,$dom);
				}else{
					$newElement->nodeValue = $value;
				}
			}else{
				if(is_array($value)){
					$newElement = $dom->createElement($key);
					$currentTag->appendChild($newElement);
					self::arrayElementsToXML($value,$newElement,$dom);
				}else{
					$newElement = $dom->createElement($key,$value);
					$currentTag->appendChild($newElement);
				}
			}

		}
	}

	public function updateVendaEnvio($nRec,$dhRecbto,$cdoperador,$nrorg,$nrseqvenda,$cdcaixa,$cdfilial){
		$dhRecbto = new \DateTime($dhRecbto);
		$fDhRecbto = $dhRecbto->format('Y/m/d');
 		$saveEnvParams = array(
			'NRRECIBONFCE' 	  => $nRec,
			'CDOPERENVIONFCE' => $cdoperador,
			'DTENVIONFCE' 	  => $this->databaseUtil->convertToDateTime($fDhRecbto),
			'NRORG'		 	  => $nrorg,
			'NRSEQVENDA'  	  => $nrseqvenda,
			'CDCAIXA'	  	  => $cdcaixa,
			'CDFILIAL'	  	  => $cdfilial
		);
		$this->entityManager->getConnection()->executeQuery(self::UPDATE_VENDA_RECIBO, $saveEnvParams);
	}

	const UPDATE_VENDA_RECIBO = "
        UPDATE VENDA
           SET NRRECIBONFCE    = :NRRECIBONFCE,
               CDOPERENVIONFCE = :CDOPERENVIONFCE,
               DTENVIONFCE     = :DTENVIONFCE,
               DTENTRVENDA     = :DTENVIONFCE
         WHERE NRORG           = :NRORG
           AND NRSEQVENDA      = :NRSEQVENDA
           AND CDFILIAL        = :CDFILIAL
           AND CDCAIXA         = :CDCAIXA
    ";

	public function updateVendaProtocolo($protocoloNota,$cdoperador,$nrorg,$nrseqvenda,$cdcaixa,$cdfilial){
		$nProt = '';
		if(isset($protocoloNota['nProt'])){
		   $nProt = $protocoloNota['nProt'];
		}
	    $msgProt = $protocoloNota['xMotivo'];
		$statusProt =  $protocoloNota['cStat'];
		$descProt = $statusProt.' - '.$msgProt;
		$savePosReceParams = array(
			'NRPROTOCOLONFCE' => $nProt,
			'DSOBSSTATUSNFCE' => $descProt,
			'NRORG'		 	  => $nrorg,
			'NRSEQVENDA'  	  => $nrseqvenda,
			'CDCAIXA'	  	  => $cdcaixa,
			'CDFILIAL'	  	  => $cdfilial
		);
		$this->entityManager->getConnection()->executeQuery(self::UPDATE_VENDA_PROTOCOLO, $savePosReceParams);
	}

	const UPDATE_VENDA_PROTOCOLO = "
        UPDATE VENDA
           SET NRPROTOCOLONFCE     = :NRPROTOCOLONFCE,
               DSOBSSTATUSNFCE     = :DSOBSSTATUSNFCE
         WHERE NRORG               = :NRORG
           AND NRSEQVENDA          = :NRSEQVENDA
           AND CDFILIAL            = :CDFILIAL
           AND CDCAIXA             = :CDCAIXA
    ";

	public function updateVendaXML($filialInfo, $nrseqvenda, $signedXMLName){
		$dsQrCode = self::getQrCode($signedXMLName);
		$dsArqXML = self::getXMLFile($signedXMLName);
		$idXML = 'S'; // --------------- VERIFICAR DEPOIS
		$updateXMLParams = array(
			'IDXMLVALIDNFCE' => $idXML,
			'DSQRCODENFCE'   => $dsQrCode,
			'DSARQXMLNFCE'   => $dsArqXML,
			'NRORG'		 	 => $filialInfo['nrorg'],
			'NRSEQVENDA'  	 => $nrseqvenda,
			'CDCAIXA'	  	 => $filialInfo['cdcaixa'],
			'CDFILIAL'	  	 => $filialInfo['cdfilial']
		);

		if ($this->util->databaseIsOracle()) {
				$this->entityManager->getConnection()->executeQuery(self::UPDATE_VENDA_XML_ORACLE, $updateXMLParams);
			} else {
				$this->entityManager->getConnection()->executeQuery(self::UPDATE_VENDA_XML_MSDE, $updateXMLParams);
			}
		return $dsQrCode;
	}

	const UPDATE_VENDA_XML_MSDE = "
            UPDATE VENDA
           SET IDXMLVALIDNFCE      = :IDXMLVALIDNFCE,
               DSQRCODENFCE        = :DSQRCODENFCE,
               DSARQXMLNFCE        = CAST(CAST(:DSARQXMLNFCE AS VARBINARY(MAX)) AS IMAGE)
         WHERE NRORG               = :NRORG
           AND NRSEQVENDA          = :NRSEQVENDA
           AND CDFILIAL            = :CDFILIAL
           AND CDCAIXA             = :CDCAIXA
    ";

    	const UPDATE_VENDA_XML_ORACLE = "
        UPDATE VENDA
           SET IDXMLVALIDNFCE      = :IDXMLVALIDNFCE,
               DSQRCODENFCE        = :DSQRCODENFCE,
               DSARQXMLNFCE        = :DSARQXMLNFCE
         WHERE NRORG               = :NRORG
           AND NRSEQVENDA          = :NRSEQVENDA
           AND CDFILIAL            = :CDFILIAL
           AND CDCAIXA             = :CDCAIXA
    ";

	Public function updateVendaStatusNfce($filialInfo, $nrseqvenda, $idStatusNfce){
		try{
			$updateSaleStatusNFCeParams = array(
				'IDSTATUSNFCE'	  => $idStatusNfce,
				'DTEMISSAONFCE'   => $this->databaseUtil->getCurrentDateTime(),
				'DTENTRVENDA'	  => $this->databaseUtil->getCurrentDate(),
				'NRORG'		 	  => $filialInfo['nrorg'],
				'NRSEQVENDA'  	  => $nrseqvenda,
				'CDCAIXA'	  	  => $filialInfo['cdcaixa'],
				'CDFILIAL'	  	  => $filialInfo['cdfilial']
			);
			$this->entityManager->getConnection()->executeQuery(self::UPDATE_VENDA_STATUS_NFCE, $updateSaleStatusNFCeParams);

			$result = array(
				'error' => false
			);
		} catch(\Exception $e){
			$result = array(
				'error' => true,
				'message' => $e->getMessage()
			);
		}
		return $result;

    }

    const UPDATE_VENDA_STATUS_NFCE = "
        UPDATE VENDA
           SET IDSTATUSNFCE        = :IDSTATUSNFCE,
               DTENTRVENDA         = :DTENTRVENDA,
               DTEMISSAONFCE       = :DTEMISSAONFCE
         WHERE NRORG               = :NRORG
           AND NRSEQVENDA          = :NRSEQVENDA
           AND CDFILIAL            = :CDFILIAL
           AND CDCAIXA             = :CDCAIXA
    ";

	public function updateTotal($property_array,&$total_xml, $icms){
		$ICMSTot = $total_xml['ICMSTot'];
		$vBCAcumulado = $ICMSTot['vBC'];
		$ICMSTot['vBC'] = 0;
		foreach($ICMSTot as $tKey=>$totalProperty){
			self::updateTotalProperty($tKey,$totalProperty, $property_array);
			$ICMSTot[$tKey] = number_format($totalProperty,2 ,'.', '');
		}

		$this->updateVBCTot($ICMSTot,$icms);

		$ICMSTot['vBC'] = number_format($ICMSTot['vBC'] + $vBCAcumulado,2 ,'.', '');
		$total_xml['ICMSTot'] = $ICMSTot;
		return $total_xml;
	}

	public function updateTotalProperty($propertyName, &$propertyValue, $arrayToCompare){
		foreach($arrayToCompare as $pKey=>$item){
			if(is_array($item)){
			   self::updateTotalProperty($propertyName,$propertyValue,$item);
			}else{
				if($propertyName == $pKey && $propertyName != 'vBC'){
					$propertyValue = $propertyValue + $item;
				}
				// else if($propertyName == $pKey && $propertyName == 'vBC'){
				// 	$propertyValue = $item;
				// }
			}
		}
	}

	public function updateVBCTot(&$ICMSTot,$icms){
		$icmsArray  = array_values($icms);
		$icmsNode  = $icmsArray[0];
			if(isset($icmsNode['vBC'])){
			$ICMSTot['vBC'] += (double)$icmsNode['vBC'];
		}
	}
	public function formataCaracteresXML($strXML){
		$formattedXML = strtr(
	            $strXML,
     			array (
			      'À' => 'A', 'Á' => 'A', 'Â' => 'A', 'Ã' => 'A', 'Ä' => 'A', 'Å' => 'A',
      			  'Æ' => 'A', 'Ç' => 'C', 'È' => 'E', 'É' => 'E', 'Ê' => 'E', 'Ë' => 'E',
      			  'Ì' => 'I', 'Í' => 'I', 'Î' => 'I', 'Ï' => 'I', 'Ð' => 'D', 'Ñ' => 'N',
				  'Ò' => 'O', 'Ó' => 'O', 'Ô' => 'O', 'Õ' => 'O', 'Ö' => 'O', 'Ø' => 'O',
				  'Ù' => 'U', 'Ú' => 'U', 'Û' => 'U', 'Ü' => 'U', 'Ý' => 'Y', 'Ŕ' => 'R',
				  'Þ' => 's', 'ß' => 'B', 'à' => 'a', 'á' => 'a', 'â' => 'a', 'ã' => 'a',
				  'ä' => 'a', 'å' => 'a', 'æ' => 'a', 'ç' => 'c', 'è' => 'e', 'é' => 'e',
				  'ê' => 'e', 'ë' => 'e', 'ì' => 'i', 'í' => 'i', 'î' => 'i', 'ï' => 'i',
				  'ð' => 'o', 'ñ' => 'n', 'ò' => 'o', 'ó' => 'o', 'ô' => 'o', 'õ' => 'o',
				  'ö' => 'o', 'ø' => 'o', 'ù' => 'u', 'ú' => 'u', 'û' => 'u', 'ý' => 'y',
				  'þ' => 'b', 'ÿ' => 'y', 'ŕ' => 'r'
    			)
			  );
		return $formattedXML;
	}

	public function isComb($cfop){
		$isComb = false;
		if($cfop =='1651' || $cfop =='1652' || $cfop =='1653' || $cfop =='1658' || $cfop =='1659' ||
		   $cfop =='1660' || $cfop =='1661' || $cfop =='1662' || $cfop =='1663' || $cfop =='1664' ||
		   $cfop =='2651' || $cfop =='2652' || $cfop =='2653' || $cfop =='2658' || $cfop =='2659' ||
		   $cfop =='2660' || $cfop =='2661' || $cfop =='2662' || $cfop =='2663' || $cfop =='2664' ||
		   $cfop =='3651' || $cfop =='3652' || $cfop =='3653' || $cfop =='5651' || $cfop =='5652' ||
		   $cfop =='5653' || $cfop =='5654' || $cfop =='5655' || $cfop =='5656' || $cfop =='5657' ||
		   $cfop =='5658' || $cfop =='5659' || $cfop =='5660' || $cfop =='5661' || $cfop =='5662' ||
		   $cfop =='5663' || $cfop =='5664' || $cfop =='5665' || $cfop =='5666' || $cfop =='5667' ||
		   $cfop =='6651' || $cfop =='6652' || $cfop =='6653' || $cfop =='6654' || $cfop =='6655' ||
		   $cfop =='6656' || $cfop =='6657' || $cfop =='6658' || $cfop =='6659' || $cfop =='6660' ||
		   $cfop =='6661' || $cfop =='6662' || $cfop =='6663' || $cfop =='6664' || $cfop =='6665' ||
		   $cfop =='6666' || $cfop =='6667' || $cfop =='7651' || $cfop =='7654' || $cfop =='7667') {
				$isComb = true;
		}

		return $isComb;
	}

	public function getIdStatusNfce($protocoloNota){
		$statusProt =  $protocoloNota['cStat'];
		$idStatus = 'R';

		switch($statusProt){
			case '100':
			case '150':
				$idStatus = 'A';
				break;
			case '105':
				$idStatus = 'T';
				break;
			case '215':
				$idStatus = 'R';
			case '110':
			case '301':
			case '302':
			case '303':
			case '999':
				$idStatus = 'D';
				break;
			default:
				$idStatus = 'R';
				break;
		}
		return $idStatus;
	}

	public function updateVendaDadosNFCe($filialInfo, $nrseqvenda, $nfceInfo, $tpAmb){
		$updateSaleNFCeParams = array(
        	'CDFILIAL'		 => $filialInfo['cdfilial'],
        	'NRSEQVENDA' 	 => $nrseqvenda,
        	'CDCAIXA'	 	 => $filialInfo['cdcaixa'],
        	'NRORG'		 	 => $filialInfo['nrorg'],
        	'CDSERIENFCE'    => $nfceInfo['serieCH'],
			'IDTPAMBNFCE'	 => $tpAmb,
			'NRACESSONFCE'   => $nfceInfo['nrAcessoNFCE'],
			'NRNOTAFISCALCE' => $nfceInfo['nrNFE'],
			'NRLANCTONFCE'   => $nfceInfo['randomNrNFE']
        );

        $this->entityManager->getConnection()->executeQuery(self::UPDATE_VENDA_DADOSNFCE, $updateSaleNFCeParams);
    }

    const UPDATE_VENDA_DADOSNFCE = "
      UPDATE VENDA
         SET CDSERIENFCE    = :CDSERIENFCE,
             IDTPAMBNFCE    = :IDTPAMBNFCE,
             NRACESSONFCE   = :NRACESSONFCE,
             NRNOTAFISCALCE = :NRNOTAFISCALCE,
             NRLANCTONFCE   = :NRLANCTONFCE
       WHERE CDFILIAL       = :CDFILIAL
         AND NRSEQVENDA     = :NRSEQVENDA
         AND CDCAIXA        = :CDCAIXA
         AND NRORG          = :NRORG
    ";
}