<?php

namespace Odhen\API\Service;

class SAT {

	protected $entityManager;
	protected $util;
	protected $satXml;
	protected $satHttp;
	protected $databaseUtil;
	protected $impressaoSAT;

 	public function __construct(
 		\Doctrine\ORM\EntityManager $entityManager,
 		\Odhen\API\Util\Util $util,
 		\Odhen\API\Lib\SatXml $satXml,
 		\Odhen\API\Lib\SatHttp $satHttp,
 		\Odhen\API\Util\Database $databaseUtil,
 		\Odhen\API\Service\ImpressaoSAT $impressaoSAT) {

		$this->entityManager = $entityManager;
		$this->util = $util;
		$this->satXml = $satXml;
		$this->satHttp = $satHttp;
		$this->databaseUtil = $databaseUtil;
		$this->impressaoSAT = $impressaoSAT;
	}

	private function validaVenda($CDFILIAL, $CDCAIXA, $NRSEQVENDA, $NRORG) {
		if (!empty($CDFILIAL)) {
			if (!empty($CDCAIXA)) {
				if (!empty($NRSEQVENDA)) {
					if (!empty($NRORG)) {
						$result = array(
							'error' => false
						);
					} else {
						$result = array(
							'error' => true,
							'message' => 'Código da organização vazio.'
						);
					}
				} else {
					$result = array(
						'error' => true,
						'message' => 'Código da venda vazia.'
					);
				}
			} else {
				$result = array(
					'error' => true,
					'message' => 'Código do caixa vazio.'
				);
			}
		} else {
			$result = array(
				'error' => true,
				'message' => 'Código da filial vazio.'
			);
		}
		return $result;
	}

	private function validaDadosSAT($dadosSAT) {
		// @todo: descobrir a descição desses campos
		if (!empty($dadosSAT)) {
			if (!empty($dadosSAT['DSPRINTHOST'])) {
				if (!empty($dadosSAT['DSSATHOST'])) {
					if (!empty($dadosSAT['CDATIVASAT'])) {
						$result = array(
							'error' => false
						);
					} else {
						$result = array(
							'error' => true,
							'message' => 'CDATIVASAT não parametrizado.'
						);
					}
				} else {
					$result = array(
						'error' => true,
						'message' => 'DSSATHOST não parametrizado.'
					);
				}
			} else {
				$result = array(
					'error' => true,
					'message' => 'DSPRINTHOST não parametrizado.'
				);
			}
		} else {
			$result = array(
				'error' => true,
				'message' => 'Configuração do SAT não encontrada para o caixa.'
			);
		}
		return $result;
	}

	public function vendaSAT($CDFILIAL, $CDCAIXA, $NRSEQVENDA, $NRORG, $NRINSCRCONS, $NMCONSVEND, $CDSENHAPED, $CDOPERADOR, $impostoFederal, $impostoEstadual, $simulaImpressao) {
		$validacaoVenda = self::validaVenda($CDFILIAL, $CDCAIXA, $NRSEQVENDA, $NRORG);
		if ($validacaoVenda['error'] == false) {
			$dadosSAT = self::carregaDadosSAT($CDFILIAL, $CDCAIXA, $NRORG);
	    	$respostaImpressora = $this->satHttp->iniciarImpressora($dadosSAT['CDSAT']);
	    	if ($respostaImpressora['error'] == false) {
				$respostaConsulta = $this->satHttp->consultarSAT();
				if ($respostaConsulta['error'] == false) {
					// @TODO remover
					$dadosSAT['CDATIVASAT'] = $this->util->getXMLParameter('CDATIVASAT');
					$dadosSAT['DSPRINTHOST'] = $this->util->getXMLParameter('URL_NODE_SERVER');
					$dadosSAT['DSSATHOST'] = $this->util->getXMLParameter('URL_NODE_SERVER');

					$respostaValidacaoSAT = self::validaDadosSAT($dadosSAT);
					if ($respostaValidacaoSAT['error'] == false) {
						$CDATIVASAT = $dadosSAT['CDATIVASAT'];
						$respostaStatus = $this->satHttp->consultarStatusOperacional($CDATIVASAT);
						if ($respostaStatus['error'] == false) {
							$respostaXML = $this->satXml->montaXML($CDFILIAL, $CDCAIXA, $NRSEQVENDA, $NRORG);
							if ($respostaXML['error'] == false) {
								$xml = $respostaXML['xml'];
								$respostaVenda = $this->satHttp->enviarDadosVenda($CDATIVASAT, $xml);
								if ($respostaVenda['error'] == false) {
									$respostaCriacaoArquivo = $this->satXml->criaArquivoSAT(
										$CDFILIAL,
										$CDCAIXA,
										$respostaVenda['arquivoCFeBase64'],
										$respostaVenda['NRACESSONFCE']
									);
									if ($respostaCriacaoArquivo['error'] == false) {
										$DTEMISSAONFCE = $this->databaseUtil->getCurrentDateTime();
										$resultadoAtualizacao = self::atualizaVendaSAT(
											$CDFILIAL,
											$CDCAIXA,
											$NRSEQVENDA,
											$NRORG,
											$respostaVenda['DSQRCODENFCE'],
											$respostaVenda['NRACESSONFCE'],
											$respostaCriacaoArquivo['DSARQXMLNFCE'],
											$respostaCriacaoArquivo['NRNOTAFISCALCE'],
											$respostaCriacaoArquivo['NRLANCTONFCE'],
											$respostaCriacaoArquivo['CDSERIESAT'],
											$respostaCriacaoArquivo['IDTPAMBNFCE'],
											$DTEMISSAONFCE,
											$CDOPERADOR
										);
										if ($resultadoAtualizacao['error'] == false) {
											if ($simulaImpressao) {
												$resultadoImpressaoCupom = array('error' => false);
											} else {
												$resultadoImpressaoCupom = $this->impressaoSAT->imprimeCupomNF(
													$CDFILIAL,
													$CDCAIXA,
													$NRORG,
													$NRSEQVENDA,
													$respostaCriacaoArquivo['NRNOTAFISCALCE'],
													$respostaCriacaoArquivo['CDSERIESAT'],
													$DTEMISSAONFCE,
													$respostaVenda['NRACESSONFCE'],
													$NRINSCRCONS,
													$NMCONSVEND,
													$CDSENHAPED,
													$respostaVenda['DSQRCODENFCE'],
													$impostoFederal,
													$impostoEstadual
												);
											}
											if ($resultadoImpressaoCupom['error'] == false) {
												$result = array(
													'error' => false,
													'xml' => $respostaCriacaoArquivo['DSARQXMLNFCE']
												);
											} else {
												$result = $resultadoImpressaoCupom;
											}
										} else {
											$result = $resultadoAtualizacao;
										}
									} else {
										$result = $respostaCriacaoArquivo;
									}
								} else {
									$result = $respostaVenda;
								}
							} else {
								$result = $respostaXML;
							}
						} else {
							$result = $respostaStatus;
						}
					} else {
						$result = $respostaValidacaoSAT;
					}
				} else {
					$result = $respostaConsulta;
				}
	    	} else {
	    		$result = $respostaImpressora;
	    	}
		} else {
			$result = $validacaoVenda;
		}

		return $result;
	}

    private function carregaDadosSAT($CDFILIAL, $CDCAIXA, $NRORG) {
    	$params = array(
			'CDFILIAL' => $CDFILIAL,
			'CDCAIXA' => $CDCAIXA,
       		'NRORG' => $NRORG
		);
		return $this->entityManager->getConnection()->fetchAssoc(self::BUSCA_DADOS_SAT, $params);
    }

    const BUSCA_DADOS_SAT = "
        SELECT DSPRINTHOST, DSSATHOST, CDATIVASAT, CDSAT
          FROM CAIXA
         WHERE CDFILIAL = :CDFILIAL
           AND CDCAIXA = :CDCAIXA
           AND NRORG = :NRORG
    ";

	private function atualizaVendaSAT($CDFILIAL, $CDCAIXA, $NRSEQVENDA, $NRORG, $DSQRCODENFCE,
	                                  $NRACESSONFCE, $DSARQXMLNFCE, $NRNOTAFISCALCE, $NRLANCTONFCE,
	                                  $CDSERIESAT, $IDTPAMBNFCE, $DTEMISSAONFCE, $CDOPERADOR) {
		try {
			$params = array(
				'IDSTATUSNFCE'   => 'A',
				'IDORIGEMVENDA'  => 'BAL_SAT',
				'DSQRCODENFCE'   => $DSQRCODENFCE,
				'NRACESSONFCE'   => $NRACESSONFCE,
				'NRNOTAFISCALCE' => $NRNOTAFISCALCE,
				'NRLANCTONFCE'   => $NRLANCTONFCE,
				'DSARQXMLNFCE'   => $DSARQXMLNFCE,
				'CDSERIESAT'     => $CDSERIESAT,
				'IDTPAMBNFCE'    => $IDTPAMBNFCE,
				'DTENTRVENDA'    => $this->databaseUtil->getCurrentDate(),
				'DTEMISSAONFCE'  => $DTEMISSAONFCE,
				'CDFILIAL' 		 => $CDFILIAL,
				'CDCAIXA' 		 => $CDCAIXA,
				'NRSEQVENDA'     => $NRSEQVENDA,
				'NRORG'          => $NRORG,
				'CDOPERULTATU'   => $CDOPERADOR
			);

			if ($this->util->databaseIsOracle()) {
				$this->entityManager->getConnection()->executeQuery(self::ATUALIZA_VENDA_SAT_ORACLE, $params);
			} else {
				$this->entityManager->getConnection()->executeQuery(self::ATUALIZA_VENDA_SAT_MSDE, $params);
			}

			$result = array(
				'error' => false
			);
		} catch (\Exception $e) {
			$result = array(
				'error' => true,
				'message' => $e->getMessage()
			);
		}
		return $result;
	}

    const ATUALIZA_VENDA_SAT_ORACLE = "
        UPDATE VENDA
           SET IDSTATUSNFCE = :IDSTATUSNFCE,
               IDORIGEMVENDA = :IDORIGEMVENDA,
               DSQRCODENFCE = :DSQRCODENFCE,
               NRACESSONFCE = :NRACESSONFCE,
               NRNOTAFISCALCE = :NRNOTAFISCALCE,
               NRLANCTONFCE = :NRLANCTONFCE,
               DSARQXMLNFCE = :DSARQXMLNFCE,
               CDSERIESAT = :CDSERIESAT,
               IDTPAMBNFCE = :IDTPAMBNFCE,
               DTENTRVENDA = :DTENTRVENDA,
               DTEMISSAONFCE = :DTEMISSAONFCE,
               CDOPERULTATU = :CDOPERULTATU,
               NRORGULTATU = :NRORG
         WHERE CDFILIAL = :CDFILIAL
           AND CDCAIXA = :CDCAIXA
           AND NRSEQVENDA = :NRSEQVENDA
           AND NRORG = :NRORG
    ";

    const ATUALIZA_VENDA_SAT_MSDE = "
        UPDATE VENDA
           SET IDSTATUSNFCE = :IDSTATUSNFCE,
               IDORIGEMVENDA = :IDORIGEMVENDA,
               DSQRCODENFCE = :DSQRCODENFCE,
               NRACESSONFCE = :NRACESSONFCE,
               NRNOTAFISCALCE = :NRNOTAFISCALCE,
               NRLANCTONFCE = :NRLANCTONFCE,
               DSARQXMLNFCE = CAST(CAST(:DSARQXMLNFCE AS VARBINARY(MAX)) AS IMAGE),
               CDSERIESAT = :CDSERIESAT,
               IDTPAMBNFCE = :IDTPAMBNFCE,
               DTENTRVENDA = :DTENTRVENDA,
               DTEMISSAONFCE = :DTEMISSAONFCE,
               CDOPERULTATU = :CDOPERULTATU,
               NRORGULTATU = :NRORG
         WHERE CDFILIAL = :CDFILIAL
           AND CDCAIXA = :CDCAIXA
           AND NRSEQVENDA = :NRSEQVENDA
           AND NRORG = :NRORG
    ";

}