describe("CalendarController", function () {
	var listController;
	var metaDataFactory;
	var zhPromise;
	var $scope;
	var $rootScope;
	var screenService;
	var container;
	var widget;
	var field;

	var baseContainer = ZhHelper.baseContainerWithTwoWidgets;
	var baseDataSourceData = ZhHelper.getTestDataSourceDataWithNames;
	var calendarField = function () {
		return {
			"id": "28684119420607610711252",
			"name": "DATE",
			"label": "Select a date",
			"template": "field/calendar.html",
			"isVisible": true,
			"fields": [],
			"events": [],
			"dateField": "DATE",
			"titleField": "TITLE",
			"dateFormat": "YYYY-MM-DD",
			"dataSource": {
				"data": [
					{
						"ID": 1,
						"TITLE": "Reunião 1",
						"DATE": "2015-05-26"
					},
					{
						"ID": 2,
						"TITLE": "Reunião 2",
						"DATE": "2015-05-27"
					},
					{
						"ID": 3,
						"TITLE": "Reunião 3",
						"DATE": "2015-05-28"
					},
					{
						"ID": 4,
						"TITLE": "Reunião 4",
						"DATE": "2015-06-01"
					}
				]
			},
			"validations": {
				"required": true
			}
		};
	};

	beforeEach(function () {
		module(function ($provide) {
			//Providing a new $timeout because $timeout.flush is not flushing pending $timeout calls
			$provide.constant("$timeout", setTimeout);
			$provide.constant("$q", Q);
		});
		module("ZeedhiApp");
		module("ZeedhiServices");

		inject(function (_$rootScope_, $controller, _ScreenService_, _metaDataFactory_, _ZHPromise_) {
			metaDataFactory = _metaDataFactory_;
			screenService = _ScreenService_;
			zhPromise = _ZHPromise_;
			container = baseContainer();
			container.widgets[0].template = "widget/form-edit.html";
			container.widgets[0].fields = [calendarField()];
			container = metaDataFactory.containerFactory(container);
			widget = container.widgets[0];
			field = widget.fields[0];
			$rootScope = _$rootScope_;

			$scope = _$rootScope_.$new();
			$scope.$parent = _$rootScope_;
			$scope.field = widget.fields[0];

			listController = $controller("CalendarController", {
				$scope: $scope
			});
		});
	});

	it("currentRow watch: Should set the field's value according to the specified dateFormat", function () {
		widget.currentRow = {};
		$scope.$apply();
		expect(field.value()).not.toBeDefined();
		field.value("2016-08-30 00:00:00");
		$scope.$apply();
		expect(field.value()).toEqual("2016-08-30");
	});
	
	it("currentRow watch: Should set the field's formattedValue according to the displayFormat", function () {
		widget.currentRow = {};
		field.displayFormat = "DD/MM/YYYY";
		field.value("2016-08-30 00:00:00");
		$scope.$apply();
		expect(field.formattedValue).toEqual("30/08/2016");
	});
	
	it("clearField: should clear the field's value and formattedValue and set the currentDate to the current one.", function () {
		widget.currentRow = {};
		field.value("2016-08-30 00:00:00");
		$scope.clearField(field);
		expect(field.value()).toEqual("");
		expect(field.formattedValue).toEqual("");
		expect(field.currentDate).toEqual(moment().startOf('month').startOf('day'));
	});

});