## Odhen Menu Component
Component created to list and manage your menu items. Works well on mobile and desktop devices :)

## Installation
**1)** First of all, check if you have Private Bower's Registry registered in your **.bowerrc** file.
```js
{
    "registry": {
        "search": [
            "http://192.168.122.56:5678/"
        ]
    }
}
```
**2)** Then install via Bower:
```bash
$ bower install --save odh-menu
```
**3)** After installed, you need to link the main component's JavaScript and CSS files in your index.html:
```html
<script src="bower_components/odh-menu/dist/odh-menu.js">
<link rel="stylesheet" href="bower_components/odh-menu/dist/odh-menu.css" />
```
**4)** Configure the package for usign the component inside your Zeedhi project:
```javascript
{
    package: "odh-menu",
    baseUrl: "bower_components/odh-menu/dist/template/"
},
```

### Using as a widget in your Zeedhi Project
After you installed the component, create a widget using the package created inside your json file for the screen, like below:
```javascript
{
    "name": "menu",
    "label": "Cardápio",
    "isVisible": true,
    "template": "odh-menu#menu.html",
    "class": "12",
    "events": [],
    "menuProperties": {
        "showGroupImages": false,
        "showProductImageOnMenu": false,
        "showCustomObservation": true,
        "colors": {
            "group": {
                "text": "#fafafa",
                "background": "#b75520",
                "opened": {
                    "background": "#e4976e",
                    "text": "#fafafa"
                }
            }
        },
        "events": {
            "addToCart": "ScreenService.showMessage('Adding product to cart!')"
        }
    },
    "actions" : [
        {
            "name": "cart",
            "isVisible": true,
            "showAsAction" : "always",
            "icon": "cart",
            "readOnly": false,
            "events": [
                {
                    "name": "ActionEvent",
                    "code": "ScreenService.showMessage('Your cart action')"
                }
            ]
        }
    ]
}
```
## Properties
As I showed above, the odh-menu component has a menuProperties property to handle all the properties and interations on your component.

Parent properties:

| Property name | Type | Default | Description |
|---------------|------|---------|-------------|
| showGroupImages | boolean | n/a | Shows parent groups with images |
| showProductImageOnMenu | boolean | n/a | Show image on products |
| showCustomObservation | boolean | n/a | Shows custom observation button on selected product popup |
| events | object | n/a | Events object |

### Events
Handle all events and interactions with the component.

| Property name | Type | Default | Description |
|---------------|------|---------|-------------|
| addToCart | string | n/a | Fired when you click on "Add" button inside selected product popup |

## MenuService Interface
During the development of the component inside a project, I figured out the need of methods to be called 'outside' the component, so this methods are inside MenuService.

Just inject the MenuService on your controller, like below:
```javascript
function YourController(MenuService) { 
    ... 
}
```

Methods:
- addToCart:
    - Generic method to add item to cart, handles if product is combo, normal or from a combo 
    - @param {Widget} menuWidget - Not much to say, just the menu widget

- getTotalPrice:
	 - Get total price of items in cart

- getCart
    - Returns the cart
    
- updateCartHint
    - Updates the hint on cart action with the cart items length
    - @param {Widget} menuWidget - Not much to say, just the menu widget

- clearCart
    - Clear cart items

- removeCartItem
    - Remove item from cart
    - @param {String} cartItemCode - Item key

- updateCart
    - Overrides current cart with cart passed as parameter
    - @param {Array} cart

- getProductObservations
    - Gets only selected observations from product
    - @param {Object} item - current selected item

- getTotalCurrentComboProducts
    - @param {Array} products - products array

If there's a need to create another method, please feel free to contribute or contact me :)

## Contribuidores
Luiz Baldi